#!/usr/bin/perl -w
# +========================================================================+
# || CAT CipUX Administration Tool                                        ||
# || CAT-web module: netgroupadmin.cgi                                    ||
# ||                                                                      ||
# || (c) 2006 Benedikt Wildenhain                                         ||
# || License: GPLv2 or later                                              ||
# +========================================================================+

package netgroupadmin_cgi;

# +=======================================================================+
# ||                                                                     ||
# +=======================================================================+
use CGI::FormBuilder qw($jshead);
use lib ".";
use CipUX::CAT::Web;
use strict;

# +=======================================================================+
# ||                                                                     ||
# +=======================================================================+
# GLOBAL
our $script = 'netgroupadmin';
our $login   = '';
our $ticket  = '';
our $cookie1 = '';
our $cookie2 = '';
our ($path,$lang,$theme,$query,$server) = &init();

&debug("================================= start cipux cat web netgroupadmin  ==============");
&debug("path [$path]\nlang [$lang]\ntheme [$theme]\nquery [$query]\nserver [$server]\n");

# +=======================================================================+
# ||                                                                     ||
# +=======================================================================+

&debug("- get cookies");
$ticket=$query->cookie(-name=>'cat_session');
$login=$query->cookie(-name=>'cat_login');

&debug("- check_for_login");
($login, $ticket, $cookie1, $cookie2) = &check_for_login($login,$ticket);

&debug("- renew_session");
($login, $ticket, $cookie1, $cookie2) = &renew_session($login,$ticket);

&debug("- print cgi header");
print $query->header(-cookie=>[$cookie1,$cookie2]);

# +=======================================================================+
# ||                                                                     ||
# +=======================================================================+
# initialize forms after ticket is known

&debug("- initialize forms");
my $idx = &idx;
my $create = &create;
my $entries = &entries(0);

&debug("- print html header");
print &header;

&debug("- check for ACL access");
&check_access($login,$ticket,$script);

&debug("- print status line");
print &statusline($login,$ticket,$script);

&debug("FORM1 S [".$idx->submitted."]") if defined $idx->submitted;
&debug("FORM1 V [".$idx->validate."]") if defined $idx->validate;

if ($entries->submitted and $entries->field('entry')) { #&& $entries->validate 

  my $action=$create->submitted;
  my $entry=$entries->field('entry');
  my $gid = $entries->field('gid') || die "No Netgroup id!";
  #warn "$action -- $gid -- $entry";
  if ($action eq 'delete' or $action eq 'edit') {
    my ($host, $user, $domain) = (split /,/, substr $entry, 1, -1);
    &rpc('str','cipux_task_remove_triple_from_netgroup',[$login,$ticket,$gid, $host, $user, $domain]);
    print &tpl("result/delete_object",{object=>"$gid: $entry"});
  }
  if ($action eq 'create' or $action eq 'edit') {
    my ($host, $user, $domain) = ($entries->field('host'), $entries->field('user'), $entries->field('domain'));
    &rpc('str','cipux_task_add_triple_to_netgroup',[$login,$ticket,$gid, $host, $user, $domain]);
    print &tpl("result/create_object",{object=>"$gid: ($host, $user, $domain)"});
  }
}

# +=======================================================================+
# ||                                                                     ||
# +=======================================================================+
if ($idx->submitted && $idx->validate) {
  # DELETE
  if($idx->submitted eq 'delete' and not $entries->field('entry')){
    my $gid = $idx->field('gid') || die "No netgroup id!";
    &debug("* delete object [$gid]");
    &rpc('str','cipux_task_delete_netgroup',[$login,$ticket,$gid]);
    print &tpl("result/delete_object",{object=>$gid});
  }
  # EDIT
  elsif ($create->submitted eq 'edit' or $entries->field('entry')){
    print &tpl("netgroupadmin/entry_header"); 
    my $gid = $idx->field('gid') || die "No Netgroup id!";
    my $entry_ref = $server->call('cipux_task_list_triple_members_of_netgroup',@{[$login,$ticket,$gid]});
    foreach my $entry (sort @{$entry_ref}){
      &debug( "- [$entry]");
      $entries->field(name => 'gid', type =>'hidden', value => $gid, force=>1);
      $entries->field(name => 'entry', type =>'hidden', value => $entry, force=>1);
      my ($host, $user, $domain) = (split /,/, substr $entry, 1, -1);
      $entries->field(name => 'host', value => "$host", force=>1);
      $entries->field(name => 'user', value => "$user", force=>1);
      $entries->field(name => 'domain', value => "$domain", force=>1);
      print $entries->render();
    }
    $entries = &entries(1);
    $entries->field(name => 'gid', type =>'hidden', value => $gid, force=>1);
    $entries->field(name => 'entry', type =>'hidden', value => '-', force=>1);
    $entries->field(name => 'host', value => '-', force=>1);
    $entries->field(name => 'user', value => '-', force=>1);
    $entries->field(name => 'domain', value => '-', force=>1);
    print $entries->render();
  }

}


&debug("FORM2 S[".$create->submitted."]") if defined $create->submitted;
&debug("FORM2 V[".$create->validate."]") if defined $create->validate;

if ($create->submitted and not $entries->field('entry')) { #&& $create->validate) {
  # CREATE
  if($create->submitted eq 'create'){
    my $id = $create->field('id') || exception('2-000-010-0003-004','no_gid_given');
    &debug("* create object [$id]");
    &rpc('str','cipux_task_create_netgroup',[$login,$ticket,$id]);
    print &tpl("result/create_object",{object=>$id});
  } }

# +=======================================================================+
# ||                                                                     ||
# +=======================================================================+
# PRINT idx table
print &tpl("netgroupadmin/idx_header"); 
# why this can't be a sub call rcp?

my $group_ref = $server->call('cipux_task_list_netgroups',@{[$login,$ticket]});
foreach my $group (sort @{$group_ref}){
  &debug( "- [$group]");
  $idx->field(name => 'gid', type =>'static', value => "$group",force=>1);
  print $idx->render();
}
print &tpl("netgroupadmin/idx_footer"); 

# PRINT ceate table
&debug("* print create table");
print &tpl("netgroupadmin/create_header"); 
print $create->render();
print &tpl("netgroupadmin/create_footer"); 

# +=======================================================================+
# ||                                                                     ||
# +=======================================================================+
print &footer(0);
exit(0);

# +=======================================================================+
# || local sub                                                           ||
# +=======================================================================+

# new object
sub create {
  my $create = CGI::FormBuilder->new(
             method => 'post',
             fields => [qw/id/],
	     validate => {
                           id => /[A-Za-z-]+/,
	                 },
             required => 'ALL',
	     submit => [qw/create/],
	     params => $query,
	     sticky =>0,
	     keepextra => 1,
	     stylesheet => "$path/$lang/$theme/form.css",
	     template => "$path/$lang/$theme/header.html",
	     template => "$path/$lang/$theme/netgroupadmin/create_body.html",
        );

  return $create;
}

# index
sub idx {
  my $idx = CGI::FormBuilder->new(
             method => 'post',
             fields => [qw/gid/],
	     validate => {gid => /[A-Za-z-]+/},
	     require  => 'gid',
             submit => [qw/edit delete/],
	     params => $query,
	     keepextra => 1,
	     table => 0, 
	     reset => 0,
	     sticky => 0,
	     stylesheet => "$path/$lang/$theme/form.css",
	     template => "$path/$lang/$theme/netgroupadmin/idx_body.html",
        );

$idx->field(name => 'gid', type=> 'static');
#$idx->field(name => 'password', type=> 'password', value=>"");
#$idx->field(name => 'password_repeat', type=> 'password', value=>"");
 return $idx;
}

# entries
sub entries {
  my $new_entry = shift;
  my $entries = CGI::FormBuilder->new(
             method => 'post',
             fields => [qw/gid entry host user domain/],
	     validate => {
	     	gid => "/[A-Za-z0-9][A-Za-z0-9-]*/",
		host => "/[A-Za-z0-9][A-Za-z0-9-]*/", # - also allowed
		domain => "/[A-Za-z0-9][A-Za-z0-9.-]*/", # dito
		},
	     require  => 'gid',
             submit => ($new_entry?[qw/create/]:[qw/edit delete/]),
	     params => $query,
	     keepextra => 1,
	     table => 0, 
	     ($new_entry?'':reset => 0),
	     sticky => 0,
	     stylesheet => "$path/$lang/$theme/form.css",
	     template => "$path/$lang/$theme/netgroupadmin/entry_body.html",
        );

$entries->field(name => 'gid', type=> 'static');
#$idx->field(name => 'password', type=> 'password', value=>"");
#$idx->field(name => 'password_repeat', type=> 'password', value=>"");
 return $entries;
}

