#!/usr/bin/perl -w
# +========================================================================+
# || CAT CipUX Administration Tool                                        ||
# || CAT-web module: useradmin.cgi                                        ||
# ||                                                                      ||
# || (c) 2006 Christian Kuelker                                           ||
# || License: GPLv2 or later                                              ||
# +========================================================================+
package useradmin_cgi;

# +=======================================================================+
use CGI::FormBuilder qw($jshead);
use CipUX::CAT::Web;
use strict;

# GLOBAL
our $script = 'useradmin';
our $login   = '';
our $ticket  = '';
our $cookie1 = '';
our $cookie2 = '';
our ($path,$lang,$theme,$query,$server) = &init();

&debug("================================= start cipux cat web useradmin  ==============");

# +=======================================================================+

$ticket=$query->cookie(-name=>'cat_session');
$login=$query->cookie(-name=>'cat_login');

($login, $ticket, $cookie1, $cookie2) = &check_for_login($login,$ticket);
($login, $ticket, $cookie1, $cookie2) = &renew_session($login,$ticket);

print $query->header(-cookie=>[$cookie1,$cookie2]);

my $idx = &idx;
my $create_user = &create_user;

print &header;

&check_access($login,$ticket,$script);

print &statusline($login,$ticket,$script);

# +=======================================================================+
if ($idx->submitted && $idx->validate) {
  # SET
  if($idx->submitted eq 'set'){
    my $oid = $idx->field('oid')             || 
      exception('2-000-010-0001-001','no_uid_given');
    my $p1  = $idx->field('password')        || 
      exception('2-000-010-0002-002','no_password_given');
    my $p2  = $idx->field('password_repeat') || 
      exception('2-000-010-0002-003','no_password_given');
    if($p1 eq $p2){
      &rpc('str','cipux_task_change_user_password',[$login,$ticket,$oid,$p1]);
    }else{
      exception('2-000-010-0004-008','passwords_not_equal');
    } 
    print &tpl("result/change_password",{object=>$oid});
  }
  # DELETE
  if($idx->submitted eq 'delete'){
    my $oid = $idx->field('oid') || die "No user id!";
    rpc('str','cipux_task_delete_user',[$login,$ticket,$oid]);
    print &tpl("result/delete_object",{object=>$oid});
  }
}

if ($create_user->submitted && $create_user->validate) {
  # CREATE
  &debug("* create user was called");
  if($create_user->submitted eq 'create'){
   
   my $field_href = $create_user->field;
   foreach(sort keys %$field_href){
     &debug(" field [$_] value $$field_href{$_}");
   }

    &debug(" * create user was submitted");
    my $id = $create_user->field('id')                            || 
      exception('2-000-010-0003-004','no_oid_given');
    &debug("ID [$id]");

    my $firstname    = $create_user->field('firstname')           || 
      exception('2-000-011-0011-008','no_firstname_given');
    my $lastname     = $create_user->field('lastname')            || 
      exception('2-000-011-0012-009'.'no_lastname_given');
    #my $role     = $create_user->field('role')                || 
    #  exception('2-000-011-0013-010','no_role_given');
    my $role = "student";
    my $p1           = $create_user->field('new_password')        || 
      exception('2-000-010-0002-005','no_password_given');
    my $p2           = $create_user->field('new_password_repeat') || 
      exception('2-000-010-0002-006','no_password_given');
    my $CID  = $create_user->field('CID')         || 
      exception('2-000-011-0014-011','no_default_CID_given');
   #my @CID=();
   #push(@CID,"student");
 #   my @snd_skel_Uid = $create_user->field('snd_skel_Uid')        || 
 #     exception('2-000-011-0015-012','no_snd_skel_Uid_given');
  my @snd_skel_Uid =();
  push(@snd_skel_Uid,"student");
    if($p1 eq $p2){
      &debug(" * password was equal, user will be created");
      # cipux_task_create_user UID firstname lastname role password 
      #    default_CID snd_skel_Uid uhquota usquota ghquota gsquota
      &rpc('str','cipux_task_create_user',
           [$login,$ticket,$id,$firstname,$lastname,
	    $role,$p1,$CID,$snd_skel_Uid[0]]);
    }else{
      &exception('2-000-010-0004-007','passwords_not_equal');
    }
    print &tpl("result/create_object",{object=>$id});
  }
}

# +=======================================================================+
# PRINT idx table
print &tpl("useradmin/idx_header"); 
my $user_ref = $server->call('cipux_task_list_users',@{[$login,$ticket]});
foreach my $user (sort @{$user_ref}){
  $idx->field(name => 'oid', type =>'static', value => "$user",force=>1);
  print $idx->render();
}
print &tpl("useradmin/idx_footer"); 

# PRINT ceate table
my @create = qw(student teacher guest);
print &tpl("useradmin/create_header"); 
foreach (@create){
  print &tpl("useradmin/create_".$_."_header"); 
  $create_user = &create_user($_);
  print $create_user->render();
  print &tpl("useradmin/create_".$_."_footer"); 
}
print &tpl("useradmin/create_footer"); 

print &footer(0);
exit(0);

# +=======================================================================+
# || local sub                                                           ||
# +=======================================================================+

# new object
sub create_user {
  my $object = shift;
  my $create_user = CGI::FormBuilder->new(
             method => 'post',
             fields => [qw/id 
			firstname
			lastname
			new_password
			new_password_repeat/],
	     validate => {
                           id => "NAME",
			   firstname => "NAME",
			   lastname => "NAME",
                           new_password => "NAME",
                           new_password_repeat => "NAME",
	                 },
             required => 'ALL',
	     submit => [qw/create/],
	     params => $query,
	     sticky =>0,
	     keepextra => 1,
	     stylesheet => "$path/$lang/$theme/form.css",
	     template => "$path/$lang/$theme/header.html",
	     template => "$path/$lang/$theme/useradmin/create_".$object."_body.html",
        );

  $create_user->field(name => 'new_password', type => 'password');
  $create_user->field(name => 'new_password_repeat', type => 'password');

  if($object eq 'student'){
    $create_user->field(name => 'role', type => 'hidden', value => 'student');
    # class
    my $class_aref = &rpc('aref','cipux_task_list_courses',[$login,$ticket]);
    $create_user->field(name => 'CID', 
                        type => 'select',
			options => \@$class_aref);

    # template directory
    my $skel_aref = &rpc('aref','cipux_task_list_skels',[$login,$ticket]);
    my $pos = 0;
    foreach (@$skel_aref){
      last if $_ eq 'student'; 
      $pos++;
    }
    splice(@$skel_aref,$pos,1);
    $create_user->field(name => 'snd_skel_Uid', 
                        type => 'select',
			options => \@$skel_aref);
  }
  if($object eq 'teacher'){
    $create_user->field(name => 'role', type => 'hidden', value => 'teacher');
}
  if($object eq 'guest'){
    $create_user->field(name => 'role', type => 'hidden', value => 'guest');
  }
  # other: address town email-address
  $create_user->field(name => 'cipuxAddress', type => 'input');
  $create_user->field(name => 'cipuxTown', type => 'input');
  $create_user->field(name => 'cipuxMailAddress', type => 'input');

  return $create_user;
}

# index
sub idx {
  my $idx = CGI::FormBuilder->new(
             method => 'post',
             fields => [qw/oid password password_repeat/],
	     validate => {oid => 'NAME'},
	     require  => 'oid',
             submit => [qw/set restore re-create delete/],
	     params => $query,
	     keepextra => 1,
	     table => 0, 
	     reset => 0,
	     sticky =>0,
	     stylesheet => "$path/$lang/$theme/form.css",
	     template => "$path/$lang/$theme/useradmin/idx_body.html",
        );

  $idx->field(name => 'oid', type=> 'static');
  $idx->field(name => 'password', type=> 'password', value=>"");
  $idx->field(name => 'password_repeat', type=> 'password', value=>"");
 return $idx;
}

