# +========================================================================+
# || CipUX::CAT::Web                                                      ||
# ||                                                                      ||
# || Defines cipux-cat-web subroutines                                    ||
# ||                                                                      ||
# || (c) Copyright, Urheberrecht 2006 by Christian Kuelker                ||
# ||                                                                      ||
# || License: GPL version 2 or later                                      ||
# ||                                                                      ||
# || Revision: 2006-10-01                                                 ||
# ||                                                                      ||
# +========================================================================+

# History:
# 2006-10-01 ckuelker: init
# 2006-10-30 ckuelker: cleanup

package CipUX::CAT::Web;

use 5.006000;
use strict;
use warnings;

use vars qw(@future @farfuture %cipux);

require Exporter;

our @ISA = qw(Exporter);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration       use CipUX::CAT::Web ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(

) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
#user - course
#uid  - gid (group)

our @EXPORT = qw(
                &renew_session
		&do_logout
                &check_access
                &check_for_login
                &debug
		&exception
                &footer
                &header
                &init
		&rpc
                &statusline
                &tpl
                 );

our $VERSION = '3.002011';

=head1 NAME

CipUX::CAT::Web - Perl extension for CipUX

=head1 SYNOPSIS

  use CipUX::CAT::Web;

=head1 DESCRIPTION

Provides the functions for cipux_task_*.

=head2 EXPORT

See CipUX::CAT::Web EXPORT variable.

=cut


# +========================================================================+
# ||                                                                      ||
# +========================================================================+
use CGI qw(-cookie);
use Frontier::Client;
use HTML::Template;
our $DEBUG = 1;

# ###############
# TODO: put his in to config file!
my $lang = "en";
my $theme ="default";
my $path = "/var/www/cipux/cat/tpl";
my $https_url = "https://localhost:8000/RPC2";
# ###############
my $query = new CGI;
my $server = Frontier::Client->new(url => $https_url);

sub init {
 return $path,$lang,$theme,$query,$server;
}

# +========================================================================+
# ||                                                                      ||
# +========================================================================+
sub debug {
  my $m = shift;
  system("echo '$m' >> /tmp/cipux.log") if $DEBUG;
}

# +========================================================================+
# ||                                                                      ||
# +========================================================================+
sub header {
  my $file = "$path/$lang/$theme/js-head.javascript";
  if(-e $file){
    &debug("* source $file");
    open(F,$file) or &exception('2-000-001-0005-001','can_not_open_file_for_reading',$file);
    my @f = <F>;
    close(F);
    my $js = join('',@f);
    #&debug("JS :[$js]");
    my $header =  &tpl("header");
    #&debug("TPL :[$header]");
    
    $header =~ s/<cipux_var js-head>/$js/g;
    return $header;
  }

}

# +========================================================================+
# ||                                                                      ||
# +========================================================================+
sub footer {
  my $mode  = shift;
  my $footer = "";
  if($mode){ # with back
    $footer = &tpl("footer2",{script => "groupadmin.pl"});
  }else{
    $footer = &tpl("footer1");
  }
  return $footer;
}

# +========================================================================+
# ||                                                                      ||
# +========================================================================+
sub tpl {
  my $tpl         = shift ; # template name
  my $param_h_ref = shift;
  &debug("  * tpl called");
  &debug("    - tpl [$tpl] ") if defined($tpl);
  &debug("    - param_h_ref [$param_h_ref]") if defined($param_h_ref);

  # de/default/footer.html
  my $file = "$path/$lang/$theme/$tpl.html";
  my $file_bak = $file;
 #  print "TEMPLATE file [$file]";
  my $template = 0;
  if(! -e $file){
    $file = "$path/en/default/$tpl.html";
    if(! -e $file){
       print"File [$file_bak] not found!";
      $file = "$path/en/default/error/template.html";
    }
  }
  &debug("    - file [$file]");
  $template = HTML::Template->new(filename => $file);
  $template->param($param_h_ref) if defined $param_h_ref;
  return $template->output;
}


# +========================================================================+
# ||                                                                      ||
# +========================================================================+
sub error {
  my $nr  = shift;
  my $tpl = shift;
  &debug(" nr [$nr], tpl [$tpl]");
  print &tpl($tpl,{number =>$nr});
}

# +========================================================================+
# ||                                                                      ||
# +========================================================================+
sub exception {
  my $number  = shift;
  my $tpl     = shift;
  my $info1   = shift;
  my $info2   = shift;
  my $info3   = shift;

  &debug("SUB (exception)");
  &debug("  - number [$number] title [$tpl]");
  &debug("  - info1 [$info1]") if defined($info1);
  &debug("  - info2 [$info2]") if defined($info2);
  &debug("  - info3 [$info3]") if defined($info3);

  my $form = CGI::FormBuilder->new(
             method => 'post',
             fields => [qw/title number message/],
             submit => [qw/OK/],
             params => $query,
             keepextra => 1,
             reset => 0,
             sticky =>0,
             stylesheet => "$path/$lang/$theme/form.css",
             template => "$path/$lang/$theme/exception/$tpl.html",
        );

  $form->field(name=>'number',type=>'static',value=>"$number",force=>1);
  $form->field(name=>'info1',type=>'static',value=>"$info1",force=>1) if defined($info1);
  $form->field(name=>'info2',type=>'static',value=>"$info2",force=>1) if defined($info2);
  $form->field(name=>'info3',type=>'static',value=>"$info3",force=>1) if defined($info3);
  print  $form->render();

  print &footer(0);
  exit(1);
}


# +========================================================================+
# ||                                                                      ||
# +========================================================================+
sub rpc {
  my $mode = shift; # str, aref (array  reference)
  my $cmd = shift;
  my $param_aref = shift;
  my $result;

  &debug("* EXEC [$cmd]");

  #print "<hr noshade>"; 
  my $result_ref = $server->call($cmd,@{$param_aref}) 
    or &exception('2-000-001-0009-002','execution_of_xmlrpc_cmd_failed',$cmd,$!);
  my $error = 0;
  $error = $! if defined($!);
  
   # print "REF[$result_ref]<br>";
  foreach(@{$result_ref}){
   #  print "RESULT=[$_]<br>";
   #    $error = $error + $_ if scalar($_);
    $result .= $_;
   }
 # print "<hr noshade>"; 
 # 
  &debug("* 2-000-001-008 error_while_cmd_execution [$cmd] [$error]") if $error;
  &exception('2-000-001-0008-003','error_while_cmd_execution',$cmd,$error) if $error;
  &debug("=> result[$result]");
  return $result if $mode eq 'str';
  return $result_ref if $mode eq 'aref';
}

# z========================================================================+
# ||                                                                      ||
# +========================================================================+
sub renew_session {
  &debug("  * renew_session called");
  my $login=shift;
  my $ticket =shift;
 
  my $cookie1 = '';
  my $cookie2 = '';
 
 
  &debug("login [$login] ticket [$ticket]");
  $ticket = $server->call('session',@{[$login,$ticket]}) 
     or &login('1-000-010-0011-007',"error/session_expired");
        
  $cookie1= $query->cookie(-name=>'cat_session', -value=>"$ticket");
  $cookie2 = $query->cookie(-name=>'cat_login', -value=>"$login");
  &debug("login [$login] ticket [$ticket]");
  return($login,$ticket,$cookie1,$cookie2);
 
}
# z========================================================================+
# ||                                                                      ||
# +========================================================================+

sub check_for_login {
  &debug("  * check_for_login called");
  my $login = shift;
  my $ticket = shift;

  my $cookie1 = '';
  my $cookie2 = '';

  if(not defined($ticket) or not defined($login)){
    $ticket = '';
    $login = '';
  }
  
  if($ticket eq '' or $login eq ''){
    # login page
    &debug("  - not logged in at all, display login screen!");
  
    &login();

  }elsif($ticket eq 'test' and $login eq 'test'){
    # test login
    &debug("  - user tries to log in");
    my $try_password = $query->param('try_password');
    my $try_login =$query->param('try_login');
    my  $test_ticket = '';
    if(defined($try_password) and defined($try_login)){
      $test_ticket = $server->call('login',@{[$try_login,$try_password]}) 
        or &login('1-000-010-0010-008',"error/wrong_login_or_password");
    }
    if($test_ticket ne '' ){
      &debug("   test ticket ...");
      if($test_ticket ne ''){
        &debug("     ticket OK!");
        $ticket=$test_ticket;
        $login = $try_login;
        $cookie1= $query->cookie(-name=>'cat_session', -value=>"$ticket");
        $cookie2 = $query->cookie(-name=>'cat_login', -value=>"$login");
      }else{
        &login('1-000-010-0010-004',"error/wrong_login_or_password");
      }
    }else{
        &login('1-000-010-0010-005',"error/wrong_login_or_password");
    }
  }
  return $login,$ticket, $cookie1, $cookie2;
}

sub check_access {
  my $login     = shift;
  my $ticket    = shift;
  my $module    = shift; # groupadmin

  &debug("* check_access");
  my $acl_field = "Cat_ACL_".ucfirst($module); # Cat_ACL_Groupadmin
  my $feature_field = "Cat_Feature_".ucfirst($module); # Cat_Feature_Groupadmin

  # ACL
  my $cmd = 'cipux_task_tell_me_cipux_config_variable_value';
  #cipux_task_tell_me_cipux_config_variable_value CAT Cat_ACL_Groupadmin
  my $acl = rpc('str',$cmd,[$login,$ticket,'CAT',$acl_field]);
  &debug("* module acl [$acl]");
  my $role = rpc('str','cipux_task_tell_me_user_role',[$login,$ticket,$login]);
  if(not $acl =~m/$role/ and $login ne 'cipadmin'){
    &debug("* [$role] has no right to access module [$module] with acl [$acl]");
    print &tpl("module_access_denied", {module => "$module",login=>"$login"});
    print &footer(0);
    exit(0);
  }

  &debug("- check_access");
  # Feature
  if(not rpc('str',$cmd, [$login,$ticket,'CAT',$feature_field])){
    print &tpl("module_disabled",{module => "$module"});
    print &footer(0);
    exit(0);
  }

}



sub statusline {
  my $login = shift;
  my $ticket = shift;

  my $server = $server->call('ping')
    or &exception('2-000-001-0001-006','execution_of_xmlrpc_cmd_failed','ping',$!);
  
  my $status = "OK";
  $status = "NOT OK" if $ticket eq '';

  return &tpl("statusline",{server => $server,
                 login => $login,
                 role => rpc('str','cipux_task_tell_me_user_role',[$login,$ticket,$login]),
                 version => rpc('str','cipux_task_tell_me_cipux_config_variable_value',
                                [$login,$ticket,'RPC','CipUX_RPC_Version']),
                 institution => rpc('str','cipux_task_tell_me_cipux_config_variable_value',
                    [$login,$ticket,'CipUX','CipUX_Institution_Name']),
                 status => "$status",});

}

sub do_logout {
  my $login = shift;
  my $ticket = shift;
  
 $server->call('logout',@{[$login,$ticket]});
  &login();

}


# *************************************************************************
# * Auxilliary methodes go here                                           *
# *************************************************************************

=pod

=head1 Auxilliary methodes

This methodes are reserved for internal use only.

=cut

# +------------------------------------------------------------------------+
# |  login                                                                 |
# +------------------------------------------------------------------------+

=pod

=head2 login

print CGI header, header, eventually error dialog, login dialog and footer

B<Syntax:>

  &login();
  &login('error number','error_string_id');


=cut

sub login {
   my $error = shift;
   my $msg   = shift;

     my $login = CGI::FormBuilder->new(
             method => 'post',
             fields => [qw/try_login try_password/],
             submit => [qw/login/],
             params => $query,
             keepextra => 1,
             reset => 0,
             sticky =>0,
             stylesheet => "$path/$lang/$theme/form.css",
             template => "$path/$lang/$theme/login.html",
        );

    $login->field(name => 'try_password', type =>'password');

    my $cookie1= $query->cookie(-name=>'cat_session', -value=>"test");
    my $cookie2 = $query->cookie(-name=>'cat_login', -value=>"test");
    print $query->header(-cookie=>[$cookie1,$cookie2]);
    print &tpl("header"); 
    print &error($error,$msg) if defined($error) and defined($msg);
    print $login->render();
    print &footer(0);
    exit(0);
}




