#!/usr/bin/perl
# +========================================================================+
# || CAT CipUX Administration Tool                                        ||
# || Webmin module: userselfadmin.cgi                                     ||
# || for administration of CAT and CipUX                                  ||
# || (c) 2001,2002,2003,2005 by Christian Kuelker                         ||
# || License: GPLv2                                                       ||
# +========================================================================+

# History
# 2006-08-15 ckuelker: apply Cat_Userdata


package userselfadmin_cgi;

do '../web-lib.pl';
&init_config();

use CipUX::CiBot;
use CipUX::Cat;
use CipUX::Task;

my $BRAND    = "cipworx";
my $PROJECT  = "cipux";
my $SCRIPT   = "userselfadmin.cgi";
my $GET      = "/usr/lib/cipux/lal/cipux_get_value";

&log($SCRIPT,"="x76);

# +=======================================================================+
# || check if the LDAP is there                                          ||
# +=======================================================================+
if($cibot_ldap_status eq 'offline'){
  &cat_header($remote_user,\&header,\&help_search_link,\&text);
  print &cat_error(&text('_error_no_ldap_connect'));
  exit(0);
}

&cibot_get_ldap_config();
&log($SCRIPT,"- remote user [$remote_user]");

my ($result,$role) = &cibot_acl($SCRIPT,$remote_user);
&cat_header($remote_user,\&header,\&help_search_link,\&text);

# hostname, user, arrayref
print &cat_status_line(&get_system_hostname(),
                       $remote_user." ($role)",
		       [
			&text('server'),
			&text('status'),
			$cipux{Customer},
			&text('date'),
			&text('version'),
                        $ldap{CipUX}{CipUX_Institution_Name}
		       ]
		      );

# +=======================================================================+
# ||  CAT ACL check                                                      ||
# +=======================================================================+
if($result ne 'ok'){
  print "<h1>".&text('access_denied')."</h1>";
  my $error = &text('access_denied_explanation',
                   "<b>$remote_user</b>",
                   "<b>$role</b>");
  &log($SCRIPT,$error);
  print &cat_error($error);
  &cat_footer($remote_user,\&footer,$ldap{CipUX}{Cipux_Support});
  exit(0)
}

# +=======================================================================+
# || Environment                                                         ||
# +=======================================================================+
&log($SCRIPT,"- webmin: ReadParse...");
ReadParse();# webmin

my $system = $in{'system'}    if defined($in{'system'});# system

foreach(sort keys %in){
  &log($SCRIPT,"- IN: [$_] = [$in{$_}]") if defined($in{$_});
}


# =========================================================================+
#                    Change Line                                          ||
# =========================================================================+
if(defined($in{save_admin})){
  &log($SCRIPT,"- got action eq save_admin");
  &cibot_change_userdata_conf('change',$in{attribute},$in{data});
  &log($SCRIPT,"- cibot_change_userdata_conf('change',$in{attribute},$in{data})");
  &cibot_change_userdata_conf('create',$in{attribute},$in{application});
  &log($SCRIPT,"- cibot_change_userdata_conf('create',$in{attribute},$in{application})");
}


# =========================================================================+
#                    Change user data                                     ||
# =========================================================================+
# Change user data, must be after Config!
# 
if(defined($in{save_user})){
  &log($SCRIPT,"- got action eq save_user");
  &log($SCRIPT,"- the authorzation to change this attribute is [$$create_href{$in{attribute}}]");
  if($$create_href{$in{attribute}} eq 'e'){
    if($in{attribute} eq 'PASSWORD'){
      if($in{PASSWORD} eq ''){
        print &cat_print_html_error(12,&text('useradmin_error_12'));
        &log($SCRIPT,"- remote user [$remote_user] treid to change the password to an empty password");
      }else{
        if($in{PASSWORD} eq $in{PASSWORD_REPEAT}){
          &cipux_task_change_user_password('cgi',$remote_user,$in{PASSWORD});
          &log($SCRIPT,"- remote user [$remote_user] changed password successfully");
        }else{	
	  print &cat_print_html_error(13,&text('groupadmin_error_password_mismatch'));
          &log($SCRIPT,"- remote user [$remote_user] gave to password which do not match!");
        }
      }
    }else{	    
      `cipux_set_value -u '$remote_user' -e '$in{attribute}' -v '$in{$in{attribute}}'`;
      &log($SCRIPT,"- remote user [$remote_user] changed attribute [$in{attribute}] to [$in{$in{attribute}}].");
    }
  }else{
    &log($SCRIPT,"- ACCESS VIOLATION from [$remote_user]: tried to change attribute [$in{attribute}] to [$in{$in{attribute}}]! But attribute is set to [$$create_href{$in{attribute}}]");
  }
}

# =========================================================================+
#                    Config                                               ||
# =========================================================================+
my @userselfadministration = qw(h v e);
my @applicationform        = qw(m o s);

my %trans = ();
# Change
$trans{'v'}="visible";
$trans{'e'}="editable";
$trans{'h'}="hidden";
# Create
$trans{'m'}="mandatory";
$trans{'o'}="optional";
$trans{'s'}="skip";

# m = mandatory
# o = optional
# s = skip
#
# h = hidden
# v = visible
# e = editable

my $change_href = &cibot_parse_userdata_conf('change');
my $create_href = &cibot_parse_userdata_conf('create');
my $order_href  = &cibot_parse_userdata_conf('order');

# =========================================================================+
# ||                         Main                                         ||
# =========================================================================+
print "<h2>".&text('userselfadmin')." ($remote_user as $role)</h2>";

if($role eq 'root' or $role eq 'admin'){
	
  #
  # And now the admin dialog!
  # 
  &log($SCRIPT,"* admin dialog");
  print &cat_table_header(4,$tb,&text('userdata_title'));
  print &cat_table_row([1,1,1,1],
		       [$tb,$tb,$tb,$tb],
		       [&text('attribute'),
		         &text('userselfadmin'),
		         &text('application'),
		         &text('action')]);

  foreach my $c (sort { $$order_href{$a} cmp $$order_href{$b} } keys %$order_href){
    &log($SCRIPT," - attribute [$c]");
    print "<form method=post a='userselfadmin.cgi'>"  ;
    print "<input type=hidden name=attribute value=\"$c\" >";

    my $name = "ERROR";
    if(defined(&text($c))){
      $name =  &text($c);
    }else{
      $name = $c;
    }
    my $data = "<select name='data' size=1>";
    my $application = "<select name='application' size=1>";
    foreach (@userselfadministration){
      &log($SCRIPT,"  -  change (e v h) test [$$change_href{$c}] = [$_]");
      if($$change_href{$c} eq $_){ 
        $data .= "<option selected value='$_'>".&text($trans{$_})."</option>\n";
        &log($SCRIPT,"    - selected [$_] [$trans{$_}]");
      }else{
        $data .= "<option value='$_'>".&text($trans{$_})."</option>\n";
      }
    }
    foreach (@applicationform){
      &log($SCRIPT,"  -  create (m o s) test [$$create_href{$c}] = [$_]");
      if($$create_href{$c} eq $_){
        $application .= "<option selected value='$_'>".&text($trans{$_})."</option>\n";
        &log($SCRIPT,"    - selected [$_] [$trans{$_}]");
      }else{
        $application .= "<option value='$_'>".&text($trans{$_})."</option>\n";
      }
    }
    $data .= "</select>\n";
    $application .= "</select>\n";

    my $submit = "<input type=submit name=save_admin value=\"".&text('save')."\"> ";
    print &cat_table_row([1,0,0,0],
                         [$tb,$cb,$cb,$cb],
		         [$name,$data,$application,$submit]);
    print "</form>";
  }
  print &cat_table_footer ."<br>";

  print &cat_explain_message(&text('userdata_root_hint'),$cb);

}else{

  #
  # And now the user dialog!
  # 

  print &cat_table_header(3,$tb,&text('userdata_title'));
  print &cat_table_row([1,1,1],
		       [$tb,$tb,$tb,$tb],
		       [&text('attribute'),
		         &text('userselfadmin'),
		         &text('action')]);

  #foreach my $c (sort keys %$create_href){
  foreach my $c (keys %$create_href){
    print "<form method=post a='userselfadmin.cgi'>"  ;
    print "<input type=hidden name=attribute value=\"$c\" >";

    my $name = "ERROR";
    if(defined(&text($c))){
      $name =  &text($c);
    }else{
      $name = $c;
    }
    if($$create_href{$c} ne 'h'){
      my $value = `$GET -u '$remote_user' -e $c`;
      chomp($value);
      my $data = '';
      if( $c eq 'PASSWORD'){
        $data = "<input type=\"password\" value=\"\" name=\"PASSWORD\" size=\"25\" maxlength=\"256\" /> ";
        $data .= &text('please_repeat');
        $data .=" <input type=\"password\" value=\"\" name=\"PASSWORD_REPEAT\" size=\"25\" maxlength=\"256\" />" if $c eq 'PASSWORD';
      }else{	      
        $data = "<input type=\"text\" value=\"$value\" name=\"$c\" size=\"80\" maxlength=\"256\" />";
      }
      my $submit = "<input type=submit name=save_user value=\"".&text('save')."\"> ";
      if($$create_href{$c} eq 'v'){
        $data = $value ;
        $submit ='';
      }
      print &cat_table_row([1,0,0],
                         [$tb,$cb,$cb],
		         [$name,$data,$submit]);
    }
    print "</form>";
  }
  print &cat_table_footer ."<br>";

}


# +=======================================================================+
# || Footer                                                              ||
# +=======================================================================+
&cat_footer($remote_user,\&footer,$ldap{CipUX}{Cipux_Support});

exit(0);
