#!/usr/bin/python
# -*- coding: iso-8859-15 -*-
"""Module docstring.
 *========================================================================*
 || cipux_add_userlist_to_course                                         ||
 || This script reads a list of students (firstname/lastname/course)     ||
 || and tries to migrate the old userbase.                               ||
 ||                                                                      ||
 || Copyright: (c) 2005  by Georg Damm               all Rights reserved!||
 || Licence:    GPLv2 (GPL version 2 or later)                           ||
 ||                                                                      ||
 || Revision:  2006-10-24                                                ||
 ||                                                                      ||
 *========================================================================*

This programm adds a list of users to a course

usage: cipux_add_userlist_to_course -g course -f infile 

where infile contains _all_ students of the course by

firstname, lastname , (class|course)

"""

# Logging Information
# Python 2.3:
import logging



from time import *
import getopt
from CipuxTools import *

class Usage(Exception):
    def __init__(self, msg):
        self.msg = msg

def main(argv=None):
    # parse command line options
    if argv is None:
        argv = sys.argv
    try:
        try:
            opts, args = getopt.getopt(sys.argv[1:], "hvf:g:", ["help"])
        except getopt.error, msg:
             raise Usage(msg)

        # option processing
        for option, value in opts:
            if option == "-v":
                verbose = True
            if option in ("-h", "--help"):
                raise Usage(__doc__)
            if option in ("-g", "--group"):
                courseName=value
            if option in ("-f", "--file"):
                fileStudents=value

    except Usage, err:
        print >>sys.stderr, sys.argv[0].split("/")[-1] + ": " + str(err.msg)
        print >>sys.stderr, "    for help use --help"
        return 2

    try:
        fileStudents
    except:
        print __doc__
	sys.exit(2)
    else:
        pass

    try:
        courseName
    except:
        print __doc__
	sys.exit(2)
    else:
        pass

    scriptName=sys.argv[0].split("/")[-1]
    Globals.LOGGER = createLog('/var/log/cipux/'+scriptName)

    ## create course:
    ## ToDo: is there already a group with this name?
    cmd1='cipux_task_create_course '+courseName+' '+getRandomPassword()
    cmd1Log='cipux_task_create_course '+courseName+' '+'xxxxxxxx'

    Globals.LOGGER.debug(cmd1Log)
    Globals.LOGGER.info(getCommandOutput(cmd1))

    ## get student data from the file:
    studentsList=csv2List(fileStudents)

    newStudentData=[]
    for studentData in studentsList:
        studentData[0]=convert2Ascii(studentData[0],'utf8')
        studentData[1]=convert2Ascii(studentData[1],'utf8')
        studentData[1]=re.sub('[ ]','_',studentData[1])
	if len(studentData)==3:
            studentData[2]=convert2Ascii(studentData[2],'utf8').lower()
        Globals.LOGGER.debug(str(studentData))

	if len(studentData)==3:
            newStudentData.append(NewUser(studentData[0],studentData[1],studentData[2]))
	elif len(studentData)==2:
            newStudentData.append(NewUser(studentData[0],studentData[1]))
	else:
            sys.exit(2)
    
    
## collect student data from ldap
    ##    oldStudentData=[] 
    cmd='cipux_task_list_students'
    studentUids=getCommandOutput(cmd).strip('\n').split("\n")
    oldStudentDic={}
    for studentUid in studentUids:
        Globals.LOGGER.debug( studentUid )
        cmd='cipux_get_value -u '+studentUid+' -e cipuxFirstname'
        firstName=getCommandOutput(cmd).strip("\n")
        cmd='cipux_get_value -u '+studentUid+' -e cipuxLastname'
        lastName=getCommandOutput(cmd).strip("\n")
        Globals.LOGGER.debug( str(studentUid+firstName+lastName))
        oldStudentDic[studentUid]=OldUser(studentUid,firstName,lastName) 
        
    ## Add users to course
    for newStudent in newStudentData:
        newStudentMatchedOld=[]
        for oldUserId in oldStudentDic.keys():
            if newStudent==oldStudentDic[oldUserId]:
                Globals.LOGGER.info( "bereinstimmung: "+str(newStudent)+str(oldStudentDic[oldUserId]) )
		newStudentMatchedOld.append(oldUserId)
        number=len(newStudentMatchedOld)
        if number==0:
            ## this seems to be a new user/student
            ## newStudent.addUser()
            print "Error: user is not in the database: "+str(newStudent)
            sys.exit(2)
        elif number==1:
            ## the name of the student is matched by one old student
            uid=newStudentMatchedOld[0]
            oldStudentDic[uid].addUserToCourse(courseName)
        else:
            ## the name of the student is matched by more an one old student
            ## so the user must make his choice
            print newStudent  
            Globals.LOGGER.info( "user:")
            Globals.LOGGER.info(str( newStudent))
            for nr in range(len(newStudentMatchedOld)):
                Globals.LOGGER.info(str( nr)+": "+str(oldStudentDic[newStudentMatchedOld[nr]]))
                print nr,": ", oldStudentDic[newStudentMatchedOld[nr]]
            choice=input("make your choice now: ")
            while not(choice in range(len(newStudentMatchedOld))): 
                choice=input("wrong choice. Try again: ")
            Globals.LOGGER.info("make your choice now: "+str(choice))
            uid=newStudentMatchedOld[choice]
            oldStudentDic[uid].addUserToCourse(courseName)

if __name__ == "__main__":
    sys.exit(main())
