#!/usr/bin/python
# -*- coding: iso-8859-15 -*-
"""Module docstring.
 *========================================================================*
 || cipux_move_up_students                                               ||
 || This script reads a list of students (firstname/lastname/course)     ||
 || and tries to migrate the old userbase.                               ||
 ||                                                                      ||
 || Copyright: (c) 2005-2006  by Georg Damm          all Rights reserved!||
 || Licence:    GPLv2 (GPL version 2 or later)                           ||
 ||                                                                      ||
 || Revision:  2006-10-24                                                ||
 ||                                                                      ||
 *========================================================================*

This programm moves up all students at the beginning of a new term.
Your groups must follow a naming-convention:

A main group for every student must match [0-9][0-9]*[A-z]*
All other groups must not match the regular expression!

usage: cipux_move_up_students -f filename

where filename contains _all_ students by

firstname, lastname , main course/group

"""

from time import *
import getopt
from CipuxTools import *

def main():
    # parse command line options
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hf:", ["help"])
    except getopt.error, msg:
        print  msg
        print  "for help use --help"
        sys.exit(2)
    # process options
    for o, a in opts:
        if o in ("-h", "--help"):
            print    __doc__
            sys.exit(0)
        if o=='-f':
                filenameStudents=a

    try:
        filenameStudents
    except:
        print __doc__
        sys.exit(2)


    scriptName=sys.argv[0].split("/")[-1]
    Globals.LOGGER = createLog('/var/log/cipux/'+scriptName)

    
    # Read ldap courses 
    Globals.LOGGER.info('Reading groups from ldap')
    test=getCommandOutput('cipux_task_list_courses')[:-1].split('\n')
    Globals.LOGGER.debug(str(test))
    currentLdapGroups=getCommandOutput('cipux_task_list_courses')[:-1].split('\n')
    Globals.LOGGER.debug(str(currentLdapGroups))
    
    #select only the MainGroups!!!
    for group in currentLdapGroups:
        if len(re.findall(rexCourse,group))==0:
            Globals.LOGGER.debug("no main group: "+str(group))
            currentLdapGroups.remove(group)
	elif re.findall(rexCourse,group)[0]!=group:
	    Globals.LOGGER.debug("no main group: "+str(group))
	    currentLdapGroups.remove(group)
        else:
            Globals.LOGGER.debug("Hauptgruppe: "+str(group))
    Globals.LOGGER.debug(str(currentLdapGroups))
            
    newStudentData=[]
    ## get student data from the file:
    studentsList=csv2List(filenameStudents)
    for studentData in studentsList:
        studentData[0]=convert2Ascii(studentData[0],'utf8')
        studentData[1]=convert2Ascii(studentData[1],'utf8')
        studentData[1]=re.sub('[ ]','_',studentData[1])
        studentData[2]=convert2Ascii(studentData[2],'utf8').lower()
        #studentData[2]=convert2Ascii(studentData[2],'utf8')
        Globals.LOGGER.debug(str(studentData))
        newStudentData.append(NewUser(studentData[0],studentData[1],studentData[2]))
    
    
## collect student data from ldap
    print "Reading ldap-data. This may take a while!"
    ##    oldStudentData=[] 
    cmd='cipux_task_list_students'
    studentUids=getCommandOutput(cmd).strip('\n').split("\n")
    oldStudentDic={}
    for studentUid in studentUids:
        Globals.LOGGER.debug( studentUid )

	# get first- and lastname of the user
        cmd='cipux_get_value -u '+studentUid+' -e cipuxFirstname'
        firstName=getCommandOutput(cmd).strip("\n")
        cmd='cipux_get_value -u '+studentUid+' -e cipuxLastname'
        lastName=getCommandOutput(cmd).strip("\n")

	# get the groups of the user and extract the main groups 
        cmd='id -Gn '+studentUid
	studentGroups=getCommandOutput(cmd).strip("\n").split(" ")
        groupsMain=[]
        for studentGroup in studentGroups:
            possibleMainGroup=re.findall(rexCourse,studentGroup)
            if len(possibleMainGroup)==1:
                if possibleMainGroup[0]==studentGroup:
                    groupsMain.append(studentGroup)

        # check if the user has only one mainGroup
        if len(groupsMain)==1:
            group=groupsMain[0]
	    # store the user with exactly one main group in the dictionary
            Globals.LOGGER.debug( str(studentUid+firstName+lastName+group))
            oldStudentDic[studentUid]=OldUser(studentUid,firstName,lastName,group) 
        elif len(groupsMain)==0:
            pass
        else:
            Globals.LOGGER.warning( 'user '+str(studentUid)+' has more than one matching group:'+str(groupsMain))
            sys.exit(0)

        
    #Delete old MainGroups
    Globals.LOGGER.info("delete courses: "+str(currentLdapGroups))
    for group in currentLdapGroups:
        cmd='cipux_task_delete_course '+str(group)
        Globals.LOGGER.info("delete course "+str(group) )
        Globals.LOGGER.debug(cmd)
        Globals.LOGGER.info(getCommandOutput(cmd))

    ## Versetzung:
    for newStudent in newStudentData:
        newStudentMatchedOld=[]
        for oldUserId in oldStudentDic.keys():
            if newStudent==oldStudentDic[oldUserId]:
                Globals.LOGGER.info( "found student: "+str(newStudent)+" "+str(oldStudentDic[oldUserId]) )
                if isNewUserMainGroup(newStudent.getUserMainGroup(), oldStudentDic[oldUserId].getUserMainGroup()):
                    newStudentMatchedOld.append(oldUserId)
                else:
                    Globals.LOGGER.error( "error: "+str(newStudent.getUserMainGroup())+" "+ str(oldStudentDic[oldUserId].getUserMainGroup()))
                    Globals.LOGGER.error(str(newStudent))
                    Globals.LOGGER.error(str(oldStudentDic[oldUserId]))
                    #sys.exit(0)
        number=len(newStudentMatchedOld)
        if number==0:
            ## it is definitely a new student
            newStudent.addUser()
        elif number==1:
            ## the name of the student is matched by one old student
            ## we assume that the student only changed his class/main course
            uid=newStudentMatchedOld[0]
            oldStudentDic[uid].changeUserMainGroup(newStudent.getUserMainGroup())
            del oldStudentDic[uid]
        else:
            ## the name of the student is matched by more an one old student
            ## so the user must make his choice
            print "A new user:" 
            print newStudent  
            Globals.LOGGER.info( "A new user:")
            Globals.LOGGER.info(str( newStudent))
            for nr in range(len(newStudentMatchedOld)):
                Globals.LOGGER.info(str( nr)+": "+str(oldStudentDic[newStudentMatchedOld[nr]]))
                print nr,": ", oldStudentDic[newStudentMatchedOld[nr]]
            print    "-1: new user\n"                                    
            Globals.LOGGER.info( "-1: new user\n" )                                  
            choice=input("make your choice now: ")
            while not(choice in range(len(newStudentMatchedOld)) or choice==-1): 
                choice=input("wrong choice. Try again: ")
            Globals.LOGGER.info("make your choice now: "+str(choice))
            if choice==-1:
                newStudent.addUser()
            else:
                uid=newStudentMatchedOld[choice]
                oldStudentDic[uid].changeUserMainGroup(newStudent.getUserMainGroup())
                del oldStudentDic[uid]
                
            
    ## delete old users
    for item in oldStudentDic.items():
        item[1].deleteUser()
    
    ## ToDo: delete empty MainGroups
    
if __name__ == "__main__":
    main()
