#!/usr/bin/perl -w

# +=======================================================================+
# || cipux_refine_homedir                                                ||
# ||                                                                     ||
# || resets or restore a give user home directory with the role skel     ||
# || dir and if configured replaces values inside the homedir.           ||
# ||                                                                     ||
# || (c) 2006 Chrstian Kuelker, All rights reserverd!                    ||
# ||                                                                     ||
# || License: GPLv2 or later                                             ||
# ||                                                                     ||
# +=======================================================================+

# history:
# 2006-11-05 ckuelker: init

use strict;

=pod

=head1 NAME

cipux_refine_homedir

=head1 DESCRIPTION

This script can resync or reset a home dir of a CipUX object. 
The object can be a user, course, role, ...

=head1 SYNOPSIS

  cipux_refine_homedir mode homedirpath role UID

  mode: restore
        reset

  homedirpath: path TO the home without the 
               homedir by itself!
	       Example: /skole/tjener/home0
 
  role: The role of the user. This is also
        a skeluser. This means that
	homedirpath/role must exist!

  UID: user ID, Example: wbolte  


=head1 HISTORY

The first refine_homedir script was a shell script 
written by Martin Herweg. The last version of this scipt
was renamed and modified by Martin Herweg and Christian 
Kuelker for installation of Debian-Edu in BBS Andernach.
in early 2005.

The acual version is a Perl script.

The problem with the original script was:
 - in german
 - not good readable
 - not sufficient error handling
 - hard cooded replacements

=head1 EXAMPLE

This restores the homedir of the teacher wbolte:

  cipux_refine_homedir restore /skole/tjener/home0 teacher wbolte

=cut


my $mode        = shift;
my $path        = shift; # homedirpath
my $role        = shift; # skeluser
my $user        = shift;

&usage(1,"please give 1. parameter!") if not defined $mode;
&usage(2,"please give 2. parameter!") if not defined $path;
&usage(3,"please give 3. parameter!") if not defined $skeluser;
&usage(4,"please give 4. parameter!") if not defined $user;

&usage(5,"home directory path '$path' does not exist!") if (! -d $path);
&usage(6,"skel directory '$path/$role' does not exist!") if (! -d "$path/$role");
&usage(7,"user directory '$path/$user' does not exist!") if (! -d "$path/$user");

if($mode eq 'restore'){
  $rso = "";
}elsif($mode eq 'reset'){
  $rso = "--delete --force";
}else{
  &usage(1,"mode must be 'reset' or 'restore'");
}

# normally $user == $group, but not for examinees!
my $group = qx(id -g $user);
chomp($group);
qx(id -un $user >/dev/null 2>&1);
&usage(8,'no such user') if $? >0;

# first syncopy skel to homedir
qx(rsync -alW $rso $role $path/$user/);

# afterwards do the stuff with the users etc files
if(-d "/etc/cipux/refine.d/enabled"){
  my @dir = qx(ls -1/etc/cipux/refine.d/enabled);
  foreach(@dir){
    chomp($_);
    qx(/etc/cipux/refine.d/enabled/$_ $mode $path $role $user);
  }
}

# change owner and group recursive!
qx(chown -R $user:group $path/$user);
qx(touch $path/$user);

exit(0);

sub usage {
  my $exitcode = shift;
  my $message  = shift;
  print "USAGE:\n";
  print "      cipux_refine_homedir mode homdirpath role user\n\n";
  print "mode         = 'reset' or 'restore'\n";
  print "homedirpath  = path TO the home directory of a user and a role\n";
  print "role         = skel directiory inside homedirpath\n";
  print "user         = user directory inside homedirpath\n\n";
  print "EXAMPLE:\n\n";
  print "* This will restore the home of mister bilbo.\n";
  print "     cipux_refine_homedir restore /skole/tjener/home0 student bilbo\n";
  print "* This will wipe out (make empty!!!) the home dir of sauron.\n";
  print "     cipux_refine_homedir reset /skole/tjener/home0 teacher sauron\n";
  if(defined($message)){
    print "="x76 ."\n";
    print "$message\n";
    print "="x76 ."\n";
  exit($exitcode);
  }
}


