#!/usr/bin/perl -w
# *========================================================================*
# || cipux_task                                                           ||
# || This script is the interface to predefined CipUX tasks               ||
# ||                                                                      ||
# || (c) Copyright, Urheberrecht: 1999-2005 by Christian Kuelker          ||
# ||                              All rights reserved!                    ||
# || Licence:   GPLv2 (GPL version 2 or later)                            ||
# ||                                                                      ||
# || Revision:  2006-04-05                                                ||
# ||                                                                      ||
# *========================================================================*

=pod

=head1 NAME

cipux_task - Version <[CIPUX::VERSION]>  2006-04-05 CipUX

=head1 SYNOPSIS

	cipux_task is NOT to be used directly

=head1 ABSTRACT

This script is the interface to predefined CipUX tasks.


=head1 REQUIREMENTS

       CipUX::Task 
       File::Basename

=cut


# TODO make this work with strict (see or 11.4. Taking References to Functions)
#use strict;
use CipUX::Task;
use File::Basename;

my $DEBUG = 0;

qx(echo "=============================================================" >> /var/log/cipux/cipux_task_debug.log 2>&1);

#qx(env >>/var/log/cipux/cipux_task_debug.log 2>&1);


open(DEBUG,">>/var/log/cipux/cipux_task_debug.log") 
   or die "Can't open file '/var/log/cipux/cipux_task_debug.log' for writing!\n";

my $caller = basename($0);
print DEBUG "* cipux_task called as [$caller]\n" if $DEBUG;
my $mode = "console";
print DEBUG "* cipux_task mode is   [$mode]\n" if $DEBUG;

if($DEBUG){
  print DEBUG "* registered tasks:\n";
  foreach (sort keys %task){
    print DEBUG "- $_($task{$_})\n";
  }
}

# test for only cipux_task
if($caller ne "cipux_task"){
  print DEBUG "* Ok, we ar not cipux_task directly\n" if $DEBUG;
}else{
  my $error_msg = "ERROR: cipux_task was invoked directly!";
  $error_msg .= " Use the cipux_task_* scripts instead.\n";
  die $error_msg;
  exit(1);
}

# test for registered cipux_task_*
if(defined($task{$caller})){
  print DEBUG "* Ok, this task is registered.\n" if $DEBUG;
}else{
  print DEBUG "This task is not registered!\n";
  print DEBUG "Use one of the following:\n";
  foreach (sort keys %task){
    print DEBUG "  $_\n";
  }
  die "ERROR: $caller is an unknown task!";
}

# some globals after we are sure the task exists
my ($args_no,@dummy) = split(':',$task{$caller});
print DEBUG "* the task as [$args_no] arguments\n" if $DEBUG;
my ($man,$aux) = split(',',$args_no,2);
print DEBUG "* [$man] mandatory and [$aux] optional arguments\n" if $DEBUG;

# and now the command line arguments
my $argv = $#ARGV + 1; # number of arguments
print DEBUG "* got [$argv] arguments\n" if $DEBUG;

# lets see them ony for DEBUG
if($DEBUG){
  foreach my $arg (@ARGV){
    print DEBUG "- got arg [$arg]\n";
  }
}

# look if we can help
if(defined $ARGV[0]){ # we have 1 arg
  # test for HELP
  if($ARGV[0] eq '-h' or $ARGV[0] eq '--help'){
    &taskusage($caller);
    exit(0);
  }elsif($ARGV[0] =~ /^--.*$/){
    # if we have -- commandline options we should parse it here
    # TODO: we could write a named parameter API 
    die "ERROR: argument list starts with unknown parameter (please use --help)!\n";
  }elsif($ARGV[0] =~ /^-.*$/){ # change this regex if we have no die before!
    # if we have - commandline options we should parse it here
    # TODO: we could write a named parameter API 
    die "ERROR: argument list starts with unknown parameter (please use --help)!\n";
  }else{
    print DEBUG "- got no --help, -h  and no other named parameter, continueing ...\n" if $DEBUG;
  }
}

# do some forensic
if($man > 0 or $aux > 0){# task with args
  if($argv < $man){# to few args
    &taskusage($caller);
    die "ERROR: need $man arguments, got only $argv!\n";
  }
  if($argv > ($man+$aux)){# to many args
    &taskusage($caller);
    my $sum = $man + $aux;
    die "ERROR: need only $sum arguments, but got $argv!\n";
  }
}else{
  print DEBUG "* the task as NO arguments\n" if $DEBUG;
  if($argv > 0){ # to many, more then 0
    die "ERROR: this script do not need any arguments!\n";
  }
}

my $args = $mode . " " .join(",",@ARGV);
my $exec = "&$caller($args)";
print DEBUG "* exec subroutine as: [$exec]\n" if $DEBUG;

# or 11.4. Taking References to Functions
my @my_argv = @ARGV;
unshift(@my_argv,$mode); #adds mode in front of @ARGV
my @returned = $caller->(@my_argv);


if($#returned > -1){
  print DEBUG "TASK RETURN:\n" if $DEBUG;
  foreach (@returned){
    print  "$_\n";
    print  DEBUG "$_\n" if $DEBUG;
  }
}

close(DEBUG);

exit(0);

#===========================================================================
#==== Documentation ========================================================
#===========================================================================


=head1 SEE ALSO

See the CipUX webpage and the manual at L<http://www.cipux.org>
See the mailing list L<http://www.cipux.org/mailman/listinfo/>

=head1 AUTHOR

Christian Kuelker  E<lt>christian.kuelker@cipworx.orgE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 1999-2006 by Christian Kuelker

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307 USA

=cut

