#!/usr/bin/python
# -*- coding: iso-8859-15 -*-
"""Module docstring.
 *========================================================================*
 || cipux_Tools                                                          ||
 || ===========                                                          ||
 ||                                                                      ||
 || This modul provides functions                                        ||
 || - to convert csv-data from a file to a list                          ||
 || - to execute shell comands                                           ||
 || - to convert strings from latin1 to utf8/ascii                       ||
 || - to get random passwords                                            ||
 || - to check if a group is a possible main group of a student          ||
 ||   in the new term / year                                            ||
 ||                                                                      ||
 || and the classes                                                      ||
 || - UserData                                                           ||
 || - NewUser (CipUX-User)                                               ||
 || - OldUser (WLUS-User)                                                ||
 ||                                                                      ||
 || Copyright: (c) 2005-2006  by Georg Damm          all Rights reserved!||
 || Licence:   GPLv2 (GPL version 2 or later)                            ||
 ||                                                                      ||
 || Revision:  2006-10-11                                                ||
 ||                                                                      ||
 *========================================================================*

"""

import sys
import base64
from string import replace
import re
from random import choice
import string

# Logging Information
# Python 2.3:
import logging

def createLog(filename):
    '''
    creates a Globals.LOGGER-object
    '''
    ## ToDo:
    ## -Create a new Logger class with static methods and values 
    ##  to ensure that there is only 1 instance active and to
    ##  avoid global names ???
    ## -switch to  Python 2.4:
    ##  import logging
    ##  logging.basicConfig(level=logging.DEBUG,
    ##                    format='%(asctime)s %(levelname)-8s %(message)s',
    ##                    datefmt='%a, %d %b %Y %H:%M:%S',
    ##                    filename='/var/log/cipux/CipuxTools.log',
    ##                    filemode='w')

    log = logging.getLogger(filename)
    hdlr = logging.FileHandler(filename+'.log')
    formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
    hdlr.setFormatter(formatter)
    log.addHandler(hdlr) 
    log.setLevel(logging.INFO)
    return log

## Regular expressions for the naming conventions of a class
## We use the age group and the name of a class to uniquely 
## indentify a MainGroup
rexCourseJg='[01][0-9]'
rexCourseName='[A-z]*'
rexCourse=rexCourseJg+rexCourseName

ldapPassword=" -y /etc/cipux/ldappassword.conf "

def csv2List(datafile):
    ''' csv2List(string) -> list

    csv2List(filename) opens the file filename and 
    returns the csv-data of the file in a list.
    '''
    result=[]
    csvfile=open(datafile,'r')
    fileContent=csvfile.readlines()
    csvfile.close()
    for line in fileContent:
        ## strip last return:
        tmp=line[:-1]
        ## erase all whitespace before and after a comma
        tmp=re.sub('[ ]*[,][ ]*',',',tmp)
        ## erase double whitspaces:
        tmp=re.sub('[ ]*[ ][ ]*',' ',tmp)
        ## strip leading and trailing whitespaces
        tmp=tmp.strip()
        tmp=tmp.split(",")
        result.append(tmp)
    csvfile.close()
    return result

## a function zu get the output of a shell command
import os, popen2

def getCommandOutput(command):
    ''' getCommandOutput(string) -> string

        getCommandOutput(cmd) executes the shell command cmd
	and returns the output.
    '''
    child = os.popen(command)
    data = child.read()
    err = child.close()
    if err:
        raise RuntimeError, '%s failed w/ exit code %d' % (command, err)
    return data

def convert2Ascii(name,type='utf8'):
    ''' convert2Ascii(string, [string]) -> string

        convert2Ascii converts a string of the type type to ascii
    '''
    if re.compile("[^0-9A-z\.\ \-]+").search(name)==None:
        return name
    else:
        if type=='utf8_base64':  
            return ': '+replace(base64.encodestring(name),'\n','')
        elif type=='ascii':
            return name
        elif type=='latin1':
            coding={"":"Ae","":"Oe","":"Ue","":"ae","":"oe","":"ue","":"ss"}
            for char in coding.keys():
                name=name.replace(char,coding[char])
            return name
        elif type=='utf8':
             name=unicode(name,'latin1')
             name=name.replace(u"\xc4","Ae")
             name=name.replace(u"\xe4","ae")
             name=name.replace(u"\xd6","Oe")
             name=name.replace(u"\xf6","oe")
             name=name.replace(u"\xdc","Ue")
             name=name.replace(u"\xfc","ue")
             name=name.replace(u"\xdf","ss")
    
             name=name.replace(u"\xd4","O")
             name=name.replace(u"\xf4","o")
             name=name.replace(u"\xfb","u")
             name=name.replace(u"\xdb","U")
             name=name.replace(u"\xea","e")
             name=name.replace(u"\xca","E")
             name=name.replace(u"\xee","i")
             name=name.replace(u"\xce","I")
             name=name.replace(u"\xec","i")
             name=name.replace(u"\xeb","e")
             name=name.replace(u"\xed","i")
             name=name.replace(u"\xea","e")
             name=name.replace(u"\xe8","e")
             name=name.replace(u"\xe9","e")
             name=name.replace(u"\xe2","a")
             return name
        else:
            sys.exit(0)

def isNewUserMainGroup(newMainGroup, oldMainGroup):
    ''' isNewUserMainGroup(string, string) -> boolean

    isNewUserMainGroup(newMainGroup, oldMainGroup) checks if
    newMainGroup is possibly the new main group of a
    user with the old main group oldMainGroup.
    '''
    newGroupName=re.findall(rexCourse,newMainGroup)
    oldGroupName=re.findall(rexCourse,oldMainGroup)
   
    if len(newGroupName)==1 and len(oldGroupName)==1:
        # check if the regular expression returns the full groupname:
        if (newGroupName[0]!=newMainGroup) or (oldGroupName[0]!=oldMainGroup):
            LOGGER.info("That was not a main group")
            return False

        newJg=re.findall(rexCourseJg,newMainGroup)
        oldJg=re.findall(rexCourseJg,oldMainGroup)
        differ=int(newJg[0])-int(oldJg[0])
        if differ==0:
            Globals.LOGGER.info("student did'nt move up")
            return True
        elif differ==1:
            Globals.LOGGER.info("student moved up")
            return True
        else:
            Globals.LOGGER.debug(str(differ)+" Something is wrong!")
            return False
    else:
        Globals.LOGGER.info("That was'nt a main group")
        sys.exit(0)

         
## routines for random password generation
def GenPasswd():
    chars = string.letters + string.digits
    newpasswd=''
    for i in range(8):
        newpasswd = newpasswd + choice(chars)
    return newpasswd

def GenPasswd2(length=8, chars=string.letters + string.digits):
    return ''.join([choice(chars) for i in range(length)])

def getRandomPassword():
    return GenPasswd()

class Globals:
    LOGGER=""

class UserData:
    ''' This class contains a minimal set of common user data.
    '''
    ## Minimal set of userdatas: Firstname, Lastname, Maincourse/group
    def __init__(self,userFirstName, userLastName, userMainGroup=""):
        self.__userFirstName=convert2Ascii(userFirstName,'utf8')
        self.__userLastName=convert2Ascii(userLastName,'utf8')
        self.__userMainGroup=convert2Ascii(userMainGroup,'utf8')
      
    def __cmp__(self, other):
        if (self.__userFirstName==other.__userFirstName) and (self.__userLastName==other.__userLastName):
            return 0
        else:
            return 1
   
    def __str__(self):
##       result="Vorname: "+self.__userFirstName+"Nachname: "+self.__userLastName+"Gruppe: "+self.__userMainGroup
        result=self.__userFirstName+" "+self.__userLastName+" "+self.__userMainGroup
        return result
   
    def getUserFirstName(self):
        return self.__userFirstName

    def getUserLastName(self):
        return self.__userLastName
      
    def getUserMainGroup(self):
        return self.__userMainGroup

    def setUserMainGroup(self,newMainGroup):
        self.__userMainGroup=newMainGroup

class NewUser(UserData):
    """Module docstring.
       This class contains information and a method to create
       new CipUX users.
    """
    def addUser(self):
        """ Creates a new Cipux user.
        """
      
        ## create the new group/course if necessary
	try:
	    getCommandOutput("id -g "+self.getUserMainGroup())
	except RuntimeError:
            cmd1='cipux_task_create_course '+self.getUserMainGroup()+' '+getRandomPassword()
	    cmd1Log='cipux_task_create_course '+self.getUserMainGroup()+' '+'xxxxxxxx'
	else:
            cmd1='echo '+self.getUserMainGroup()+' already exists'
	    cmd1Log=cmd1
        
        cmd2='cipux_task_create_applicant "'+self.getUserFirstName()+'" '+self.getUserLastName()+' student '+getRandomPassword()+' '+self.getUserMainGroup()
        cmd2Log='cipux_task_create_applicant "'+self.getUserFirstName()+'" '+self.getUserLastName()+' student '+'xxxxxxxx'+' '+self.getUserMainGroup()
      
        Globals.LOGGER.debug(cmd1Log)
        Globals.LOGGER.info(getCommandOutput(cmd1))
        Globals.LOGGER.debug(cmd2Log)
        Globals.LOGGER.info(getCommandOutput(cmd2))

class OldUser(UserData):
    ''' This class contains information and methods to 
    modify and delete already existing CipUX users.
    '''
    def __init__(self,userId, userFirstName, userLastName, userMainGroup=""):
        UserData.__init__(self, userFirstName, userLastName, userMainGroup)
        self.__userId=userId
   
    def __str__(self):
##      result="Uid: "+self.__userId+" "+UserData.__str__(self)
        result=self.__userId+": "+UserData.__str__(self)
        return result
      
    def getUserId(self):
        return self.__userId

    def deleteUser(self):
        ''' Deletes an existing Cipux user.
        '''
        cmd='cipux_task_delete_user '+self.__userId
        Globals.LOGGER.info("Lsche Benutzer "+self.__userId+" Klasse "+self.getUserMainGroup() )
        Globals.LOGGER.debug(cmd)
        Globals.LOGGER.info(getCommandOutput(cmd))

    def changeUserMainGroup(self, newGroupName):
        ''' Changes the main group of an existing Cipux user.
        '''
        ## create the new group/course if necessary
	try:
	    getCommandOutput("id -g "+newGroupName)
	except RuntimeError:
            cmd1='cipux_task_create_course '+newGroupName+' '+getRandomPassword()
            cmd1Log='cipux_task_create_course '+newGroupName+' '+'xxxxxxxx'
	else:
            cmd1='echo do nothing '+newGroupName+' already exists'
            cmd1Log=cmd1
        
        cmd2='cipux_task_add_user_to_course '+newGroupName+' '+self.getUserId()
      
        Globals.LOGGER.debug( cmd1Log)
        Globals.LOGGER.info( getCommandOutput(cmd1) )
        Globals.LOGGER.debug( cmd2)
        Globals.LOGGER.info( getCommandOutput(cmd2) )
        self.setUserMainGroup(newGroupName)
   
    def addUserToCourse(self, courseName):
        ''' Adds an existing Cipux user to a existing course.
        '''
        ## create the new group/course if necessary
	try:
	    getCommandOutput("id -g "+courseName)
	except RuntimeError:
            cmd1='cipux_task_create_course '+courseName+' '+getRandomPassword()
	    cmd1Log='cipux_task_create_course '+courseName+' '+'xxxxxxxx'
	else:
            cmd1='echo do nothing '+courseName+' already exists'
            cmd1Log=cmd1
        

        cmd2='cipux_task_add_user_to_course '+courseName+' '+self.getUserId()
      
        Globals.LOGGER.debug( cmd1Log)
        Globals.LOGGER.info( getCommandOutput(cmd1) )
        Globals.LOGGER.debug( cmd2)
        Globals.LOGGER.info( getCommandOutput(cmd2) )
        self.setUserMainGroup(courseName)
