#!/usr/bin/perl -w
# +=======================================================================+
# || cipux_maint_diagnostic                                              ||
# ||                                                                     ||
# || This script runs several tests against  Test.pm and tries to        ||
# || check if the CipUX installation is ok.                              ||
# ||                                                                     ||
# ||                                                                     ||
# || (c) 2006 Christian Kuelker                                          ||
# ||                                                                     ||
# || License: GPLv2 (GPL version 2 or later)                             ||
# ||                                                                     ||
# +=======================================================================+

# History:
# 2006-08-23 ckuelker: add switches for CAT-webmin

use Test;
use File::stat;
use strict;

my $SET = "/usr/lib/cipux/lal/cipux_set_value";
my $GET = "/usr/lib/cipux/lal/cipux_get_value";

my $cipux_conf = "/etc/cipux/cipux.conf";
my $cipux_schema = "/etc/ldap/schema/cipux.schema";
my $cipux_ldappassword_conf ="/etc/cipux/ldappassword.conf";
my %package = ();

$package{'cipux-common'}=1;
$package{'cipux-cibot'}=1;
$package{'cipux-cat-webmin'}=1;
$package{'cipux-samba'}=1;
$package{'cipux-client'}=1;
$package{'cipux-notifier'}=1;
$package{'cipux-rpc'}=1;



my $customer = qx(cat /etc/cipux/customer.conf);
my $pre = shift;

if(not defined($customer)){
  print "ERROR: The name of the customer or distribution is not defined.\n";
  print "        Please add a pre defined name to the file customer.conf\n";
  print "        Possible values are debian-edu or ubuntu\n";
  print "Example: echo -n \"debian-edu\" > /etc/cipux/customer.conf\n";
  exit(1);
}
if($customer eq ""){
  print "ERROR: The variable \$customer is defined but empty.\n";
  print "        Possible values are debian-edu or ubuntu\n";
  print "Example: echo -n \"debian-edu\" > /etc/cipux/customer.conf\n";
  exit(1);
}

if($customer ne "debian-edu" and $customer ne "ubuntu"){
  print "ERROR: The variable \$customer is defined, but has a wrong value. \n";
  print "       It has the value: $customer\n";
  print "       But it should have:  debian-edu or ubuntu\n";
  exit(1);
}

my $section = "00";
my $test    = "01";

BEGIN { plan tests => 41 }

print "# This script will test the CipUX installation.\n";
print "# If somthing is not ok, follow the instruction in brackets: [...] if any\n";
print "#\n";

# +=======================================================================+
# || section 01 /etc/cipux                                               ||
# +=======================================================================+
$section++;$test="01";
print "#"."="x77 . "\n#\n";
print "#          Testing the CipUX /etc configuration:       \n#\n";

# 01
print "# [$section.$test]: exsists /etc/cipux? =>\t\t\t\t\t";
exit(1) if not ok(-d "/etc/cipux");# [please install all packages or copy the folder from an instelled CipUX]
$test++;

# 02
my $dir = "/etc/cipux";
my $info = stat($dir) or die "no $dir $!\n";
#my $mode = (stat("/etc/cipux"))[2];
my $mode = sprintf "%04o", $info->mode & 07777;
print "# [$section.$test]: /etc/cipux has the mode $mode, is this equal to 750? =>\t";
exit(1) if not ok( "0750" eq $mode); # [please execute: chmod 750  /etc/cipux]
$test++;

# 03
print "# [$section.$test]: exsists  $cipux_conf? =>\t\t\t\t";
exit(1) if not ok(-e $cipux_conf); # [please install the cipux-common package! aptitude install cipux-common]
$test++;

# 04
print "# [$section.$test]: is it readable? =>\t\t\t\t\t\t";
exit(1) if not ok(-r $cipux_conf); # [please install the cipux-common package! aptitude install cipux-common]
$test++;

# 05
print "# [$section.$test]: exists /etc/cipux/package? =>\t\t\t\t";
exit(1) if not ok(-d "/etc/cipux/package");# [please install  CipUX packages (cipux-common, cipux-cibot): aptitude install cipux-common cipux-cibot ]
$test++;

# 06
print "# [$section.$test]: exists /etc/cipux/package/cipux-common? =>\t\t\t";
exit(1) if not ok(-e "/etc/cipux/package/cipux-common");# [please install CipUX package cipux-common: aptitude install cipux-common ]
$test++;

# 07
print "# [$section.$test]: exists /etc/cipux/package/cipux-cibot? =>\t\t\t";
exit(1) if not ok(-e "/etc/cipux/package/cipux-cibot");# [please install CipUX package cipux-cibot: aptitude install cipux-cibot ]
$test++;

# 08
print "# [$section.$test]: exists $cipux_ldappassword_conf ? =>\t\t\t";
exit(1) if not ok(-e $cipux_ldappassword_conf); # [please set password: echo -n "password" > /etc/cipux/ldappassword.conf !]
$test++;

# 09
my $file = "$cipux_ldappassword_conf";
$info = stat($file) or die "no $file: $!";
print "# [$section.$test]: is $cipux_ldappassword_conf owned by user root? =>\t";
exit(1) if not ok( $info->uid == 0); # [please chown root:root /etc/cipux/ldappassword.conf !]
$test++;

#10
print "# [$section.$test]: is $cipux_ldappassword_conf owned by group root? =>\t";
exit(1) if not ok( $info->gid == 0); # [please chown root:root /etc/cipux/ldappassword.conf !]
$test++;

#11
$mode = sprintf("%04o", $info->mode & 07777);
print "# [$section.$test]: is $cipux_ldappassword_conf mode 400 ? =>\t\t";
exit(1) if not ok( $mode eq '0400'); # [please chmod 400 /etc/cipux/ldappassword.conf !]
$test++;

# 12
print "# [$section.$test]: is the Ldap password in $cipux_ldappassword_conf ? =>\t";
my $set_pwd     = qx(cat $cipux_ldappassword_conf);
chomp($set_pwd);
exit(1) if not ok($set_pwd ne '');# [please set the correct root password at Ldap password in $cipux_conf]
$test++;

# 13
print "# [$section.$test]: exists /etc/cipux/release.conf? =>\t\t\t\t";
exit(1) if not ok(-e "/etc/cipux/release.conf"); # [please set the release : echo -n "release" > /etc/cipux/release.conf !]
$test++;

# 14
my $release = qx(cat /etc/cipux/release.conf);
print "# [$section.$test]: do we have a release? ";
print "$release =>\t\t\t\t" if defined($release);
exit(1) if not ok(defined($release)); # [please set the release : echo -n "release" > /etc/cipux/release.conf !]
$test++;

# 15
my $filecustomer = qx(grep -e "^ *Customer *=:*" /etc/cipux/system.conf|sed -e "s/^.*Customer.*=//g"|sed -e "s/^ *//"|sed -e "s/ *//");
chomp($filecustomer);
print "# [$section.$test]: do we have a customer? ";
print "[".$filecustomer."] =>\t\t\t" if defined($filecustomer);
exit(1) if not ok(defined($filecustomer) and $filecustomer ne "") ; # [please set the "Customer = " in /etc/cipux/system.conf !]
$test++;

# 16
print "# [$section.$test]: is the customer ok? ";
print "[".$filecustomer."] eq [$customer] =>\t\t" if defined($filecustomer);
exit(1) if not ok($filecustomer eq $customer) ; # [please set the "Customer = " in /etc/cipux/system.conf !]
$test++;

# 17
my $uname = qx(uname -a);
if($uname =~/tjener/){
    # test for debian-edu
    print "# [$section.$test]: we are on a tjener. Is the costomer then debian-edu? =>\t";
    exit(1) if not ok($customer eq "debian-edu"); # [please set the "Customer = debian-edu" in /etc/cipux/system.conf !]
}else{
    # test for default
    print "# [$section.$test]: we are on a unknown system. Is the costomer then default? =>\t";
    exit(1) if not ok($customer eq "default"); # [please set the "Customer = default" in /etc/cipux/system.conf !]
}
 $test++;

# +=======================================================================+
# || section 02 /var/log/cipux                                           ||
# +=======================================================================+
$section++;$test="01";
print "#"."="x77 . "\n#\n";
print "#          Testing /var/log/cipux:\n#\n";

# 01
print "# [$section.$test]: exsists /var/log/cipux? =>\t\t\t\t\t";
exit(1) if not ok(-d "/var/log/cipux");# [please install all packages or copy the folder from an instelled CipUX]
$test++;

#02
$dir = "/var/log/cipux";
$info = stat($dir) or die "no $dir $!\n";
print "# [$section.$test]: is $dir owned by user root? =>\t\t\t";
exit(1) if not ok( $info->uid == 0); # [please chown root:root /var/log/cipux !]
$test++;

#03
print "# [$section.$test]: is $dir owned by group root? =>\t\t\t";
exit(1) if not ok( $info->gid == 0); # [please chown root:root /var/log/cipux !]
$test++;

# 04
$mode = sprintf "%04o", $info->mode & 07777;
print "# [$section.$test]: /var/log/cipux has the mode $mode, is this equal to 750? =>\t";
exit(1) if not ok( "0750" eq $mode); # [please execute: chmod 750  /var/log/cipux]
$test++;

# +=======================================================================+
# || section 03 /etc/ldap  LDAP Server                                   ||
# +=======================================================================+
$section++;$test="01";
print "#"."="x77 . "\n#\n";
print "#          Testing /etc/ldap and the LDAP server:\n#\n";

# 01
print "# [$section.$test]: exsists $cipux_schema? =>\t\t\t";
exit(1) if not ok(-e $cipux_schema); # [please install the cipux-common package!]
$test++;

# 02
my $include = qx(grep $cipux_schema /etc/ldap/slapd.conf);
print "# [$section.$test]: is the cipux.schema in slapd.conf included? =>\t\t";
exit(1) if not ok($include ne "");# [please edit /etc/ldap/slapd.conf and add the line: include /etc/ldap/schema/cipux.schema ]
$test++;

# 03
my $slapd = qx(ps ax|grep slapd|grep -v grep);
print "# [$section.$test]: is the LDAP server running? =>\t\t\t\t";
exit(1) if not ok($slapd); # [ please start slapd: /etc/init.d/slapd start ]
$test++;

# 04
print "# [$section.$test]: can we browse the ldap server without password? =>\t\t";
my $cmd1 ="/usr/bin/ldapsearch -x -p 389 -h localhost -b 'uid=root,ou=People,dc=skole,dc=skolelinux,dc=no' -LLL uid uid=root";
my @root1 = qx($cmd1);
chomp($root1[1]);
my $root1 = $root1[1];

if(ok($root1 eq "uid: root")){

}else{
   # print "# ERROR: we can not browse the LDAP without password!\n";
   print "# ERROR: we can not browse the LDAP without password !\n";
   print "cmd [$cmd1]\n";
   foreach (@root1){
     chomp();
     print "# * [$_]\n";
   }
   exit(1);
}
$test++;

# 05
print "# [$section.$test]: can we access the ldap server by password? =>\t\t";
my $cmd2 ="/usr/bin/ldapsearch -x -p 389 -h localhost -ZZ -y /etc/cipux/ldappassword.conf -D 'cn=admin,ou=People,dc=skole,dc=skolelinux,dc=no' -b 'uid=root,ou=People,dc=skole,dc=skolelinux,dc=no' -LLL uid uid=root";
my @root2 = qx($cmd2);
chomp($root2[1]);
my $root2 = $root2[1];
if(ok($root2 eq "uid: root")){

}else{
   #print "# ERROR: we can not access the LDAP with given password!\n";
   print "# ERROR: we can not access the LDAP with given password !\n";
   print "cmd [$cmd2]\n";
   foreach (@root2){
     chomp();
     print "# * [$_]\n";
   }
   exit(1);
}
$test++;

print "#\n# Everything looks good, you may proceed!\n";

exit (0)  if defined($pre);

# +=======================================================================+
# || section 04 CiBot Basis                                              ||
# +=======================================================================+
$section++;$test="01";
print "#"."="x77 . "\n#\n";
print "#          Testing CiBot: \n#\n";


# 01 
print "# [$section.$test]: do we have installed and registered CiBot? =>\t\t";
exit(1) if not ok(-e "/etc/cipux/package/cipux-cibot"); # cipux-cibot not installed [aptitude install cipux-cibot]
$test++;

# 02
my @modules = qw(CipUX::CiBot CipUX::Task CipUX::Conversion);
foreach my $m (@modules){
  print "# [$section.$test]: do the module '$m' load without warnings? =>\t";
  $test++;
  my $load = qx(perl -e 'use $m;' 2>&1);
  print "\n# ERROR: $load\n" if $load ne "";
  exit(1) if not ok($load eq ""); # [install a new version of the module!]
}
$test++;

# 03
print "# [$section.$test]: do the CipUX::CiBot module load with require? =>\t\t";
exit(1) if not ok(eval("require CipUX::CiBot;")); # [please install an new version of cipux-cibot!]
$test++;

# 04
print "# [$section.$test]: do the CipUX::Cat module load with require? =>\t\t";
exit(1) if not ok(eval("require CipUX::Cat;")); # [please install an new version of cipux-cat-webmin!]
$test++;

# 05
print "# [$section.$test]: do the CipUX::Task module load with require? =>\t\t";
exit(1) if not ok(eval("require CipUX::Task;")); # [please install an new version of cipux-cibot!]
$test++;

# 06
print "# [$section.$test]: do the CipUX::Conversion module load with require? =>\t";
exit(1) if not ok(eval("require CipUX::Conversion;")); # [please install an new version of cipux-cibot!]
$test++;

# +=======================================================================+
# || section 05 CIPUX functions test (roles)                             ||
# +=======================================================================+
$section++;$test="01";
print "#"."="x77 . "\n#\n";
print "#          Testing the CipUX/Cibot functions:\n#\n";

# 01
print "# [$section.$test]: is the cipux_task_list_users script installed? =>\t\t";
exit(1) if not ok(-e "/usr/bin/cipux_task_list_users");
$test++;

# 02
print "# [$section.$test]: is the cipux_task_list_users script executable? =>\t\t";
exit(1) if not ok(-x "/usr/bin/cipux_task_list_users");
$test++;

# 03
print "# [$section.$test]: get we some ouput? =>\t\t\t\t\t";
my @userlist = qx(/usr/bin/cipux_task_list_users);
exit(1) if not ok($#userlist > -1); # [ there are no CipUX users in LDAP, the cipux_setup_ldap_objects script should have failed]
$test++;


my %userlist = ();
foreach my $u (@userlist){
  chomp($u);
  $userlist{$u}=1;
  print "# * USER: $u\n";
}

# +=======================================================================+
# || section 06 CipUX default objects                                    ||
# +=======================================================================+
$section++;$test="01";
print "#"."="x77 . "\n#\n";
print "#          Testing  CipUX default objects:  \n#\n";

# 01
print "# [$section.$test]: is the user object 'cipadmin' there? =>\t\t\t";
exit(1) if not ok($userlist{cipadmin}); # [ the account cipadmin is not there, the cipux_setup_ldap_objects script should have failed. create account cipadmin!]
$test++;

# 02 
print "# [$section.$test]: is the cipux_task_list_courses script installed? =>\t\t";
exit(1) if not ok(-e "/usr/bin/cipux_task_list_courses");
$test++;

# 03
print "# [$section.$test]: is the cipux_task_list_courses script executable? =>\t\t";
exit(1) if not ok(-x "/usr/bin/cipux_task_list_courses");
$test++;

###
### There are no default courses
###

#print "# * get we some ouput? =>\t";
#my @courselist = qx(/usr/bin/cipux_task_list_courses);
#exit(1) if not ok($#userlist > 0);

#my %courselist = ();
#foreach my $c (@courselist){
#  chomp($c);
#  $courselist{$c}=1;
#  print "# * COURSE: $c\n";
#}

# 04
print "# [$section.$test]: is the cipux_task_list_roles script installed? =>\t\t";
exit(1) if not ok(-e "/usr/bin/cipux_task_list_roles");
$test++;

# 05
print "# [$section.$test]: is the cipux_task_list_roles script executable? =>\t\t";
exit(1) if not ok(-x "/usr/bin/cipux_task_list_roles");
$test++;

# 06
print "# [$section.$test]: get we some ouput? =>\t\t\t\t\t";
my @roleslist = qx(/usr/bin/cipux_task_list_roles);
exit(1) if not ok($#roleslist > 0);
$test++;

my %roleslist = ();
foreach my $r (@roleslist){
  chomp($r);
  $roleslist{$r}=1;
  print "# * ROLE: $r\n";
}


# 07
# assistent gone away in 3.2.8
print "# [$section.$test]: the role object assistent should missing.\t\t\t";
exit(1) if not ok(! $roleslist{assistent}); # [please run /usr/lib/cipux/sbin/migrate_3.2.8_to_3.2.9 !]
$test++;

# 08
print "# [$section.$test]: is the role object assistant there?\t\t\t\t";
exit(1) if not ok($roleslist{assistant}); # [please run cipux_setup_ldap_objects!]
$test++;

# 09
print "# [$section.$test]: is the role object teacher   there?\t\t\t\t";
exit(1) if not ok($roleslist{teacher}); # [please run cipux_setup_ldap_objects!]
$test++;

# 10
print "# [$section.$test]: is the role object student   there?\t\t\t\t";
exit(1) if not ok($roleslist{student}); # [please run cipux_setup_ldap_objects!]
$test++;

# 11
print "# [$section.$test]: is the role object guest     there?\t\t\t\t";
exit(1) if not ok($roleslist{guest}); # [please run cipux_setup_ldap_objects!]
$test++;

# 12
print "# [$section.$test]: is the role object examinee  there?\t\t\t\t";
exit(1) if not ok($roleslist{examinee}); # [please run cipux_setup_ldap_objects!]
$test++;

# 13
print "# [$section.$test]: is the role object tutor     there?\t\t\t\t";
exit(1) if not ok($roleslist{tutor}); # [please run cipux_setup_ldap_objects!]
$test++;

# 14
print "# [$section.$test]: is the role object none      there?\t\t\t\t";
exit(1) if not ok($roleslist{none}); # [please run cipux_setup_ldap_objects!]
$test++;


# +=======================================================================+
# || section 06 CIPUX CAT Webmin                                         ||
# +=======================================================================+
$section++;$test="01";

print "#"."="x77 . "\n#\n";
if( -e "/etc/cipux/package/cipux-cat-webmin"){
  print "# [oo]     CipUX-CAT-Webmin is installed, will test it\n#\n";

  # 01
  print "# [$section.$test]: is the CipUX Configuration there? =>\t\t\t";
  my $cipux_producer=qx($GET -x CAT -e cipuxVariable |grep Cipux_Producer);
  chomp($cipux_producer);
  $cipux_producer =~s/Cipux_Producer=//g;
  exit(1) if not ok($cipux_producer eq "www.cipworx.org");# [The CipUX Configuration LDAP nodes are missing!]
  $test++;

  # 02
  my @modules = qw(CipUX::Cat);
  foreach my $m (@modules){
    print "# [$section.$test]: do the module '$m' load without warnings? =>\t";
    $test++;
    my $load = qx(perl -e 'use $m;' 2>&1);
    print "\n# ERROR: $load\n" if $load ne "";
    exit(1) if not ok($load eq ""); # [install a new version of the module!]
  }
  $test++;

}else{
  print "# [--]     CipUX-CAT-Webmin is not installed, will not test it\n#\n";
}


# +=======================================================================+
# || section 07 CIPUX variables LDAP                                     ||
# +=======================================================================+
$section++;$test="01";
print "#"."="x77 . "\n#\n";
print "#          Testing CipUX variables in LDAP:\n#\n";

# 01
print "# [$section.$test]: is the CipUX Configuration there? =>\t\t\t\t";
my $cipux_remark=qx($GET -x CipUX -e cipuxRemark);
chomp($cipux_remark);
exit(1) if not ok($cipux_remark eq "CipUX was made by CipWorX");# Can read cipuxRemark from CipUX configuration node "CipUX" [The CipUX Configuration LDAP node CipUX  (cipuxRemark) is missing or has the wrong value! Execute: $SET -x CipUX -e cipuxRemark -v 'CipUX was made by CipWorX']

$test++;

# +=======================================================================+
# || section 08 CIPUX Samba test                                         ||
# +=======================================================================+
$section++;$test="01";
print "#"."="x77 . "\n#\n";

if( -e "/etc/cipux/package/cipux-samba"){
  print "# [oo]     CipUX-SAMBA is installed, will test it\n#\n";
  
  # 01
  print "# [$section.$test]: do we have smbpasswd? =>\t\t\t\t\t";
  exit(1) if not ok(-e "/usr/bin/smbpasswd");# there is no /usr/bin/smbpasswd, may be SAMBA was not installed?
  $test++;

  # 02
  print "# [$section.$test]: do we have objectClass sambaSamAccount in cipadmin? =>\t";
  my @object_class = qx($GET -u cipadmin -e objectClass);
  my $object_class = join(" ",@object_class);
  $object_class =~ s/\n//g;
  exit(1) if not ok($object_class =~ m/sambaSamAccount/);# cipadmin has no sambaSamAccount, CipUX was not installed with Samba support.
  $test++;


}else{
  print "# [--]     CipUX-SAMBA is not installed, will not test it\n#\n";
}



print "#"."="x77 . "\n#\n";
print "# Summary:\n";
print "#   Sections: $section\n";
foreach(sort keys %package){
  print "#  - package $_ =>\t";
  if(-e "/etc/cipux/package/$_"){
    print "yes\n";
  }else{
    print "no\n";
  }
}
print "#\n # Everything looks good, you may proceed!\n";

exit(0);

=pod

=head1 NAME

cipux_main_diagnostic - Version <[CIBOT::VERSION]>  2005-08-12 CipUX

=head1 SYNOPSIS

  cipux_main_diagnostic

=head1 ABSTRACT

This script runs several tests against  Test.pm and tries to
check if the CipUX installation is ok.

=head1 REQUIREMENTS

  CipUX::CiBot
  CipUX::Task
  CipUX::Conversion
  CipUX::Cat

=head1 SEE ALSO

See the CipUX webpage and the  manual at L<http://www.cipux.org>
See the mailing list L<http://www.cipux.org/mailman/listinfo/>

=head1 AUTHOR

Christian Kuelker  E<lt>christian.kuelker@cipworx.orgE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 1999-2006 by Christian Kuelker

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307 USA

=cut
