#!/usr/bin/perl -w
# +=======================================================================+
#||                                                                      ||
#|| cipux_setup                                                          ||
#||                                                                      ||
#|| Starts the setup process after package installation.                 ||
#||                                                                      ||
#||                                                                      ||
#||                                                                      ||
#||                                                                      ||
#||                                                                      ||
#||                                                                      ||
#|| (c) 2006 by Christian Kuelker, All rights reserverd!                 ||
#||                                                                      ||
#|| Licese: GPL version 2 or later                                       ||
#||                                                                      ||
# +=======================================================================+
#
# History:
# 2006-10-09 ckuelker initial version 
#

=pod

=head1 NAME

cipux_setup - Version <[CIBOT::VERSION]>  2005-08-01 CipUX

=head1 SYNOPSIS

        cipux_setup [OPTIONS]

        Options: (-h or -s must given) 
     (01) -s          : setup                                
     (02) -f          : force                                
     (03) -i          : interactively                        
     (04  -p          : choose a profile (interactive)       
     (05) -P PROFILE  : apply the PROFILE in batch           
     (06) -w          : disable warn dialog, saves 12 seconds
     (07) -h          : this message                         


=head1 ABSTRACT

This script starts the setup process.

 * warn about setup (if not diabled with -w)
 * checks and add ldap password (-i ask for it)
 * checks and add customer/distribution
 * test part of installtion
 * moves ou=Machines (-f force that)
 * add ou=CipUX to LDAP (-f force that)
 * add default objects and users (-f force that)
 * test more parts of the installation
 * add a profile if (-p) or (-P PROFILE)
 

=head1 REQUIREMENTS

  /etc/cipux
  Term::ReadKey

=head1 EXAMPLES

The help page:

  cipux_setup -h 

simple setup

  cipux_setup -s

mostly interactive setup (ask for passowrd and profile)

  cipux_setup -sip

force a simple setup (default user, roles recrated)

  cipux_setup -sf

force a simple setup without warning

  cipux_setup -swf                        

makes a batch setup and aplly the French school profile

  cipux_setup -swP cipux_profile_fr_school 

=cut



use Term::ReadKey;       # read chars from STDIN
use Getopt::Std;         # command line options

my $BRAND   = "cipworx";
my $PROJECT = "cipux";
my $SCRIPT  = "cipux_setup";
my $VERSION = "<[CIPUX::VERSION]>";
my $BUILD   = "<[CIBOT::BUILD]>";
my $DEBUG   = 0;

# set normal mode here
ReadMode('normal');

# Environment
my %opt = (); # for later use
# h help
# s setup
# without warning
# f force
# i interactively
# p profile
# P profile apply given profile
my $options="hswfipP:";

&debug($SCRIPT,"- options: [$options]");

getopts("$options",\%opt);

# help
if(defined($opt{h})){&usage;exit(0);}


#
&debug($SCRIPT,"- options: [$options]");

if(defined($opt{s})){ # ok, we start the setup
  if(not defined($opt{w})){ # ok we print a warning
    print "+==============================================================+\n";
    print "|| This is the setup script for CipUX!                        ||\n";
    print "||                                                            ||\n";
    print "|| Do not run this script on an used system! You will loose   ||\n";
    print "|| your setup data. Continue on your own risk (as always).    ||\n";
    print "||                                                            ||\n";
    print "|| Or press CTRL-C NOW!                                       ||\n";
    print "+==============================================================+\n";

    foreach (0..11){
      print " ($_) ";
      qx(sleep 1);
    }
    print "\n";
  }else{
    print "+==============================================================+\n";
    print "|| This is the setup script for CipUX!                        ||\n";
    print "+==============================================================+\n";
  }

  # 
  # fixes to filesystem 
  #

  # this might become its own scripte some day
  # on some architectures the package-management can't be trusted.
  qx(mkdir -p /etc/cipux/setup);
  qx(chmod 750 /etc/cipux/setup);
  qx(chown root:root /etc/cipux/setup);
  
  qx(chmod 750 /var/log/cipux);
  qx(chown root:root /var/log/cipux);

  qx(chmod 750 /etc/cipux);
  qx(chown root:root /etc/cipux);


  #
  #  ldap password
  #
  if(-e "/etc/cipux/ldappassword.conf" and not defined($opt{i})){
    print "\n";
    print "* ldappassword.conf exsists.\t\t\t\t[ OK ]\n"
  }else{
    my $password1 = undef;
    my $password2 = undef;
    do{
      print "\n";
      print "To acces the LDAP server trough the CipUX LDAP abstraction layer\n";
      print "it is nessesary to store the LDAP password into the file:\n";
      print "            /etc/cipux/ldappassword.conf\n";
      print "It will be secured by chmod 400 and chown root:root.\n";
      print "\n";
      print "Please give the LDAP password (on some sytems this is the root\n"; 
      print "password. The password will not be echoed, so it is not a\n";
      print "mistake that you will not see what you type.\n";
      print "LDAP password -> ";
      ReadMode('noecho');
      $password1 = ReadLine 0;
      ReadMode('normal');
      chomp($password1);
      print "\n";
      print "\n";
      print "Please repeat the LDAP password once again, so that we can be \n";
      print "sure you typed the password you intended to write.\n";
      print "LDAP password -> ";
      ReadMode('noecho');
      $password2 = ReadLine 0;
      ReadMode('normal');
      chomp($password2);
      print "\n";
    }until($password1 eq $password2);
    open(F,"> /etc/cipux/ldappassword.conf") or die "Can't open  /etc/cipux/ldappassword.conf!";
    print F $password1;
    close(F);
    qx(chmod 400 /etc/cipux/ldappassword.conf);
    qx(chown root:root  /etc/cipux/ldappassword.conf);
  }

  my $admin = qx(/usr/bin/ldapsearch -x -p 389 -h localhost -ZZ -y /etc/cipux/ldappassword.conf -d 0 -D 'cn=admin,ou=People,dc=skole,dc=skolelinux,dc=no'    "cn=admin" cn|grep cn:|sed -e 's%cn: %%g');
  chomp($admin);
  if($admin ne "admin"){
     print "\n" . "="x64 . "\n";
     print "ERROR: the LDAP entry 'admin' can not be read from the LDAP!\n";
     print "       (1) you LDAP server might not be running.\n";
     print "       (2) or you gave the wrong password.\n";
     print "       (3) or your LDAP server is not configured according the\n";
     print "           CipUX standard\n";
     print "You should fix this!\n";
     print "="x64 . "\n";
     exit(100);
  }

  #
  #  customer     
  #
  if(-e "/etc/cipux/customer.conf"){# TODO make that interactive
    print "\n";
    print "* customer.conf exsists.\t\t\t\t[ OK ]\n"
  }else{
    my $customer = qx(cipux_common -c);
    chomp($customer);
    if(defined($customer)){
      if($customer ne ""){
        if($customer eq "debian-edu" or $customer eq "ubuntu"){
          print " * customer as '$customer' deteted, \n";
          print "   save that to customer.conf\n";
          open(F,"> /etc/cipux/customer.conf") or die "Can't open  /etc/cipux/customer.conf!";
            print F $customer;
          close(F);
          qx(chmod 400 /etc/cipux/customer.conf);
          qx(chown root:root  /etc/cipux/customer.conf)
        }else{
          print "ERROR (unknown): cipux_common -c gives a unknown customer/distribution! \n";
          print "                 If this customer/distribution is supported, please add \n";
          print "                 it to this script and upload this script to the SVN.\n";
          exit(101);
        }
      }else{
          print "ERROR (empty): cipux_common -c faild to detect customer/distribution! \n";
          exit(102);
      }
    }else{
      print "ERROR (undef): cipux_common -c faild to detect customer/distribution! \n";
      exit(102);
    }
  }

  #
  # cipux_maint_diagnostic pre
  #
  system("/usr/lib/cipux/sbin/cipux_maint_diagnostic pre");

  #
  # moves mighty ou=Machines
  #
  if(not -e "/etc/cipux/setup/ou_machines" or defined($opt{f})){
    print "* move ou=Machines ...\n";
    system("/usr/lib/cipux/sbin/cipux_setup_ldap_machines");
    qx(date --rfc-2822 --utc > /etc/cipux/setup/ou_machines);  
  }else{
    print "* ou=Machines move setup was done before.\t\t[ OK ]\n"
  }
  #
  # adds ou=CipUX
  #
  if(not -e "/etc/cipux/setup/ou_cipux" or defined($opt{f})){
  print "* add ou=CipUX ...\n";
  system("/usr/lib/cipux/sbin/cipux_setup_ldap_configuration");
    qx(date --rfc-2822 --utc > /etc/cipux/setup/ou_cipux);  
  }else{
    print "* ou=CipUX setup was done before.\t\t\t[ OK ]\n"
  }

  #
  # add some roles and admins
  #
  if(not -e "/etc/cipux/setup/default_objects" or defined($opt{f})){
    print "* create default roles and users ...\n";
    system("/usr/lib/cipux/sbin/cipux_setup_ldap_objects");
    qx(date --rfc-2822 --utc > /etc/cipux/setup/default_objects);  
  }else{
    print "* default setup was done before.\t\t\t[ OK ]\n"
  }

  #
  # test the rest
  #
  system("/usr/lib/cipux/sbin/cipux_maint_diagnostic");


  #
  # profile
  #

  # options:
  # p                    -> choose
  # P (P=def,ok)         -> apply P
  # P (P=def,not ok)     -> choose
  # P (P=not def)        -> -
  # pP (P=def, ok)       -> apply P
  # pP (P=def, not ok)   -> choose
  # pP (P not def)       -> choose
 
  my $apply = 0;
  my $apply_name ="";

  #print "DEB [$opt{p}] [$opt{P}]\n";

  if(defined($opt{P})){
    chomp($opt{P});
    $opt{P} =~ s/^cipux_profile_//g;# should start with
    # print "DEB [$opt{p}] [$opt{P}]\n";

    # if valid cipux_profile_* script:
    if( -x "/usr/lib/cipux/sbin/cipux_profile_$opt{P}"){
        print "* valid profile\n";
        $apply = "/usr/lib/cipux/sbin/cipux_profile_$opt{P}";
        $apply_name = "cipux_profile_$opt{P}";
      }else{
        $opt{p} = 1; # switch on choosing; as a last resort.
        $apply = 0;
        print "* bach profile not valid, try choose a profile ...\n";
      }
   }
  
  if(defined($opt{p}) and not $apply){# choose if no valid, or like to choose
    my @profile = qx(cd /usr/lib/cipux/sbin && ls -1 cipux_profile_*);
    my $valid = 0;
    my $profile ="";

    do{
      print "\n";
      print "-"x64 ."\n";
      print "Profiles:\n";
      print "-"x64 ."\n";
      my $p = 0;  
      foreach (@profile){
        chomp;
        my $n = sprintf("%04d", $p);
        print "$p $_\n";
        $p++;
      }
      print "-"x64 ."\n";
      print "\n Please choose a profile => ";
      $profile = ReadLine 0;
      if($profile =~ /[0-9]/){
        if(defined($profile[$profile])){
          $valid = 1;
        }
      }
    }until($valid);
    if( -x "/usr/lib/cipux/sbin/".$profile[$profile]){
      $apply = "/usr/lib/cipux/sbin/$profile[$profile]";
      $apply_name = "$profile[$profile]";
    }
  }
  if(-x $apply){
      print "* applying profile '$apply'!\n";
      system("$apply");
      qx(date --rfc-2822 --utc > /etc/cipux/setup/$apply_name);
  }

}else{

  &usage();
  exit(1);
}

exit(0);

   # for futuer interactive mode:
   # read a char from keybord
#  ReadMode 'cbreak';
 # my $key = ReadKey(0);
  #ReadMode 'normal';
sub debug {
  if($DEBUG){
     my $s = shift; # script
     my $m = shift; # mesage
     my $v = shift; # value
     if(not  -d "/var/log/cipux"){
       system("mkdir -p /var/log/cipux && chown root:root /var/log/cipux && chmod 750 /var/log/cipux");
     }
    chomp($m);
    my $message = "$s: $m";
    $message .= " [$v]" if defined($v);
    my $file = "/var/log/cipux/".$s.".log";
    open(FILE,">>$file") or die "Can't open file [$file] for write!\n";
      print FILE $message ."\n";
    close(FILE);
  }
}

sub usage {
 print "-"x76;
  print "\n";
  print "$BRAND-$PROJECT-$SCRIPT-$VERSION \n\n";
  print "USAGE:\n\n";
  print "       $SCRIPT [OPTIONS]  \n\n";
  print "Options:\n";
  print "  (01) -s          : setup                                \n";
  print "  (02) -f          : force                                \n";
  print "  (03) -i          : interactively                        \n";
  print "  (04  -p          : choose a profile (interactive)       \n";
  print "  (05) -P PROFILE  : apply the PROFILE in batch           \n";
  print "  (06) -w          : disable warn dialog, saves 12 seconds\n";
  print "  (07) -h          : this message                         \n";
}

#===========================================================================
#==== Documentation ========================================================
#===========================================================================


=head1 SEE ALSO

See the CipUX webpage and the  manual at L<http://www.cipux.org>
See the mailing list L<http://www.cipux.org/mailman/listinfo/>

=head1 AUTHOR

Christian Kuelker  E<lt>christian.kuelker@cipworx.orgE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 1999-2006 by Christian Kuelker

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307 USA

=cut

