#!/bin/bash
# +=======================================================================+
# || cipux_setup_ldap_objects                                            ||
# ||                                                                     ||
# || The script uses CipUX and the cipux_task_* scripts to create the    ||
# || CipUX default LDAP structure concerning the default roles.          ||
# ||                                                                     ||
# ||                                                                     ||
# || (c) 2006 by Christian Kuelker                                       ||
# ||                                                                     ||
# ||  License: GPLv2 (GPL version 2 or later)                            ||
# ||                                                                     ||
# +=======================================================================+

# TODO: replace this with perl script


DELETE='jradmins students teachers machines admins'
ROLE='none student teacher tutor assistant guest examinee skel'
OTHER='course examination'
GROUP='nogroup'
SKEL='noskel'
ADMIN=cipadmin
PASSWORD=$RANDOM$RANDOM
PATH=$PATH:/usr/lib/cipux/bin:/usr/lib/cipux/sbin:/usr/lib/cipux/lal

if [ "$PASSWORD" == "" ]; then
  echo "USAGE: cipux_setup_ldap_objects DEFAULT_PASSWORD"
  exit 1
else

  if [ -d /var/log/cipux ]; then
    echo " log dir exists ..."
  else
    echo " creating log dir /var/log/cipux ..."
    mkdir -p /var/log/cipux
  fi

  # order important: first "none", second "machines"

  for i in $DELETE; do
    echo "delete wlus objects $i (if it is there, just to be save)"
    cipux_task_delete_user $i >/dev/null
  done


  for i in $ROLE; do
    echo "delete role $i (if it is there, just to be save)"
    cipux_task_delete_role $i >/dev/null
    echo "create role $i ..."
    # !!! we have to take user because refine_homedir will fail (this roles DO NOT exist)
    # cipux_task_create_object UID firstname lastname urole grole gtype password
    cipux_task_create_object   $i  CipUX     role     role  none  shared $PASSWORD
    sleep 1;
    cipux_set_value -c $i -e cipuxRole -v skel
  done

  # create the machines group for samba
  # TODO test if uid 900 exists: getent passwd 900
  cipux_task_create_object  machines CipUX     role     role  none  shared $PASSWORD
  cipux_set_value -u machines -e uidNumber -v 900
  cipux_set_value -u machines -e gidNumber -v 900
  cipux_set_value -c machines -e gidNumber -v 900
  cipux_set_value -u machines -e cipuxRole -v meta
  cipux_set_value -c machines -e cipuxRole -v skel
  cipux_set_value -c machines -e groupType -v shared
 
  if [ -x /etc/init.d/nscd ]; then /etc/init.d/nscd restart; fi
  
  if [ -e /etc/cipux/ldappassword.conf ]; then
    for i in $ADMIN; do
      echo "delete admin $i (if it is there, just to be save)"
      cipux_task_delete_user $i >/dev/null
      echo "create admin $i ..."
      # cipux_task_create_user UID firstname lastname role password
      cat /etc/cipux/ldappassword.conf | cipux_task_create_user $i Adminstrator CipUX none -
      sleep 1
      cipux_set_value -c $i -e cipuxRole -v admin
    done
  else
    echo "ERROR: /etc/cipux/ldappassword.conf is not there, can not create admin"
    echo "       please follow install guide to create this file!"
    exit 1
  fi

  for i in $SKEL; do
    echo "delete skel $i (if it is there, just to be save)"
    cipux_task_delete_skel $i >/dev/null
    echo "create skel $i ..."
    # cipux_task_create_user UID firstname lastname role password
    cipux_task_create_skel $i $PASSWORD >/dev/null 2>&1
  done


  for i in $OTHER; do
    echo "delete other role $i (if it is there, just to be save)"
    cipux_task_delete_user $i >/dev/null
    echo "create other role $i ..."
    cipux_task_create_user $i CipUX CourseRole none $PASSWORD
    sleep 1
    cipux_set_value -u $i -e cipuxRole -v meta
    cipux_set_value -c $i -e cipuxRole -v skel
    cipux_set_value -c $i -e groupType -v shared
  done

  for i in $GROUP; do
    echo "delete group $i (if it is there, just to be save)"
    cipux_task_delete_course $i >/dev/null
    echo "create group $i ..."
    # cipux_task_create_user UID firstname lastname role password
    cipux_task_create_course $i $PASSWORD >/dev/null 2>&1
  done

fi


