#!/usr/bin/perl -w
#
# +=======================================================================+
# || cipux_common                                                        ||
# ||                                                                     ||
# || Provides common information over and for CipUX                      ||
# ||                                                                     ||
# ||                                                                     ||
# ||                                                                     ||
# ||                                                                     ||
# ||                                                                     ||
# || Copyright (c) 2006 by Christian Kuelker, all Rights reserverd!      ||
# ||                                                                     ||
# || License: GPL version 2 or later                                     ||
# ||                                                                     ||
# +=======================================================================+

my $BRAND   = "cipworx";
my $PROJECT = "cipux";
my $SCRIPT  = "cipux_common";
my $VERSION = "<[CIPUX::VERSION]>";
my $BUILD   = "<[CIBOT::BUILD]>";
my $DEBUG   = 0;

=pod

=head1 NAME

cipux_common  - Version <[CIPUX::VERSION]>  2006-09-09 CipUX

=head1 SYNOPSIS

  cipux_common [OPTIONS]

  Options:
  (1) -v        : CipUX version
  (2) -r        : CipUX release
  (2) -i        : CipUX institution
  (3) -c        : CipUX customer

  Auxilliary options:
  (4) -h        :get help page

=head1 ABSTRACT
  
The cipux_common script gather information from the system or 
from the CipUX installation. The information is needed mainly
for the setup of CipUX but may also be useful for the local
system administrator.

=head1 REQUIREMENTS

  Getopt::Std

=head1 EXAMPLES

1. The command 

  cipux_common -h 

will return the help page.

2. The command 

  cipux_common -v 

will return the CipUX version number. For example: 3.2.12.

3. The command 

  cipux_common -c 

will retrun the custommers name. Mostly the distribution in this days. 
For example the return value could be "debian-edu".

4. The command 

  cipux_common -i 

will return the name of the institution or the names of institutions for
wich the CipUX installation was set up. CipUX 3.2.x supports only one
institution. But future versions will support more then one institution like 
CipUX 2.x did.

5 The command 

  cipux_common -s

tries to guess on which system this skript is executed.

=cut



# +=======================================================================+
# || Global                                                              ||
# +=======================================================================+
my $system_conf      = "/etc/cipux/system.conf";
my $institution_conf = "/etc/cipux/institution.conf";
my $release_conf     = "/etc/cipux/release.conf";

# +=======================================================================+
# || Environment                                                         ||
# +=======================================================================+

# use of environment modules
use Getopt::Std;

my %opt = (); # for later use
my $options="vrichs";

&debug($SCRIPT,"- options: [$options]");

getopts("$options",\%opt);

# help
if(defined($opt{h})){&usage;exit(0);}

# distribution/ customer
if(defined($opt{c})){
  if( -f $system_conf){
    $c = qx(cat $system_conf);
    $c =~ s/.*Customer\s*=\s*//g;
    chomp($c);
    print "$c\n";
  }else{
    print "ERROR: file [$system_conf] do not exist! CipUX setup is not complete. Please read the setup guide and create that file\n";
    exit(1);
  }
}

# institution(s)
if(defined($opt{i})){
  if( -f $institution_conf){
    $c = qx(cat $institution_conf);
    $c =~ s/.*Institution\s*=\s*//g;
    chomp($c);
    print "$c\n";
  }else{
    print "ERROR: file [$institution_conf] do not exist! CipUX setup is not complete. Please read the setup guide and create that file\n";
    exit(1);
  }
}


# version
if(defined($opt{v})){
  if( -f $release_conf){
    $c = qx(cat $release_conf);
    $c =~ s/-\d+$//g;
    chomp($c);
    print "$c\n";
  }else{
    print "ERROR: file [$release_conf] do not exist! CipUX setup is not complete. Please read the setup guide and create that file\n";
    exit(1);
  } 
} 

# release
if(defined($opt{r})){
  if( -f $release_conf){
    $c = qx(cat $release_conf);
    chomp($c);
    print "$c\n";
  }else{
    print "ERROR: file [$release_conf] do not exist! CipUX setup is not complete. Please read the setup guide and create that file\n";
    exit(1);
  }
}

# system
if(defined($opt{s})){
  if(-d "/etc/debian-edu"){
    print "debian-edu\n";
  }elsif(-d "/etc/debian"){ # TODO: write a real test!
    print "debian\n"
  }else{
    print "unknown\n";
  }
}


exit(0);

sub debug {
  if($DEBUG){
     my $s = shift; # script
     my $m = shift; # mesage
     my $v = shift; # value
     if(not  -d "/var/log/cipux"){
       system("mkdir -p /var/log/cipux && chown root:root /var/log/cipux && chmod 750 /var/log/cipux");
     }
    chomp($m);
    my $message = "$s: $m";
    $message .= " [$v]" if defined($v);
    my $file = "/var/log/cipux/".$s.".log";
    open(FILE,">>$file") or die "Can't open file [$file] for write!\n";
      print FILE $message ."\n";
    close(FILE);
  }
}

sub usage {
 print "-"x76;
  print "\n";
  print "$BRAND-$PROJECT-$SCRIPT-$VERSION \n\n";
  print "USAGE:\n\n";
  print "       $SCRIPT [OPTIONS]  \n\n";
  print "Options: (exact only one is mandatory)\n";
  print "  (01) -v        : version                         \n";
  print "  (02) -r        : release                         \n";
  print "  (03) -c        : customer                        \n";
  print "  (04) -i        : institution                     \n";
  print "  (05) -h        : this message                    \n";
}


#===========================================================================
#==== Documentation ========================================================
#===========================================================================


=head1 SEE ALSO

See the CipUX webpage and the  manual at L<http://www.cipux.org>
See the mailing list L<http://www.cipux.org/mailman/listinfo/>

=head1 AUTHOR

Christian Kuelker  E<lt>christian.kuelker@cipworx.orgE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 1999-2006 by Christian Kuelker

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
MA 02110-1301, USA.

=cut

