#!/usr/bin/perl -w
# +========================================================================+
# || /usr/lib/cipux/bot/cipux_bot_deploy_conf                             ||
# ||                                                                      || 
# || Creates shell scripts, which will be fetched by cipux clients. One   ||
# || shell script for each client. This shell script tells a client what  ||
# || image it should load.                                                ||
# ||                                                                      || 
# || Copyright: (c) 1999 - 2006  by Christian Kuelker all Rights reserved!||
# || Licence:   GPLv2                                                     ||
# ||                                                                      ||
# ||                                                                      ||
# +========================================================================+

# History
# 2006-08-17 ckuelker correct a missing 00 

=pod

=head1 NAME

deploy_conf_bot.pl - Version <[CIPUX::VERSION]>  2006-08-17 CipUX

=head1 SYNOPSIS

        cipux_bot_deploy_conf

=head1 ABSTRACT

Creates shell scripts, which will be fetched by cipux clients. One
shell script for each client. This shell script tells a client what
image it should load.

=head1 REQUIREMENTS

	CipUX::CiBot

=cut

package cipux_bot_deploy_conf;

use CipUX::CiBot;
use Getopt::Std;
use strict;

# +========================================================================+
my $PART=                  "Global VAR";#                                  |
# +========================================================================+

my $BRAND   = "cipworx";
my $PROJECT = "cipux";
my $SCRIPT  = "cipux_bot_deploy_conf";
my $GET     = "/usr/lib/cipux/lal/cipux_get_value";

my %value=();

# +========================================================================+
$PART=             "Configuration";#                                       |
# +========================================================================+

&log($SCRIPT,"="x76);

# Date as ISO-8601
#$date = actual_date();

# Time
#my $time = actual_time();
my $time = `date`;
&log($SCRIPT,"Time = $time");

# +========================================================================+
$PART=             "Environment";#                                         |
# +========================================================================+
# use of environment module
my %opt=();
getopts("Ch",\%opt);

if(defined($opt{h})){usage();exit(0)}

# +========================================================================+
$PART=             "Main Part";#                                           |
# +========================================================================+

#my @room=`cipux_task_list_rooms`;

#if($room[0]=~m/ERROR/){
#  print "ERROR connecting LDAP server, please check connection!\n";
#  print @room;
#  exit(1);
#}

my %option = ();
$option{'machine'}='M';
$option{'image_slot'}='S';
$option{'image'}='I';
$option{'hardware_type'}='T';
$option{'reference'}='I';
$option{'room'}='R';


my @plain_object = qw(machine image_slot hardware_type image);

# HoA Perl <279>
my %attribute = (
  machine => [ qw(
                  cipuxImageSlotNumber 
                  cipuxHardwareTypeNumber
                  cipuxMaster
                  cipuxClientNumber
                  cipuxRoomName
                 ) ],
  image_slot => [ qw(
                  cipuxImageSlotName
                  cipuxImageSlotProtect
                    ) ],
  hardware_type => [ qw(
                  cipuxHardwareTypeName
                       ) ],
  image => [ qw(
                  cipuxImageName
                  cipuxImageType
                  cipuxImageProtect
                  cipuxChecksum
                  cn
               ) ],
  reference => [ qw(
                  cipuxImageSlotNumber
                  cipuxHardwareTypeNumber
                    ) ],
  room => [ qw(
               cipuxImageSlotNumber
              ) ],
);

# execute every command to get LDAP values
foreach my $object(keys %attribute){
  foreach my $attribute (@{$attribute{$object}}){
    &log($SCRIPT,"object:          ($object)");# machine ...
    &log($SCRIPT,"cmd line option: ($option{$object})");
    &log($SCRIPT,"attribute:       ($attribute)");# cipuxClientNumber, ...
    my $cmd = "$GET -$option{$object} -e '$attribute'";
    &log($SCRIPT,"CMD ($cmd)");
    my @value = `$cmd`;
    foreach(@value){
      chomp();
      my($key,$value)=split("\t");
      $value{$object}{$attribute}{$key}=$value;
      &log($SCRIPT,"value:           ($object)($attribute)($key)=>($value)");
    }
#    $value{$object}{$attribute} = [ @value ];
  }
}

# get tpl file!
my $dir = "/var/lib/tftpboot/cipux/conf";
my $tpl_file = "$dir/client.tpl";
&log($SCRIPT,"processing tpl file  ($tpl_file)");
if(!-e $tpl_file){
  &log($SCRIPT,"client tpl file ($tpl_file) does NOT exist!");
  exit(1);
}
my @tpl = `cat $tpl_file`;
my $tpl = "";
foreach(@tpl){$tpl.=$_}# sum up the tpl file in one string

#foreach(sort keys %cipuxHardwareTypeName){do_log("NAME ($_)")}


# go through every machine
&log($SCRIPT,"===================process every machine======================");
foreach my $machine (keys %{ $value{'machine'}{'cipuxClientNumber'} }){
  &log($SCRIPT,"-----------------------------------------------------------------");
  &log($SCRIPT,"Process machine:          ($machine)");

  # copy to save the original
  my $tmp_tpl = $tpl;

  # generate some key values
  # save the hardware type number, the image slot number
  my %key = ();
  $key{'machine'}       = $machine;
  $key{'hardware_type'} = $value{'machine'}{'cipuxHardwareTypeNumber'}{$machine};# htn

  $key{'room'} = $value{'room'}{'cipuxImageSlotNumber'}{$value{'machine'}{'cipuxRoomName'}{$machine}};
  # we have to overwrite the Image_Slot number in machine if
  # the client is NOT a master client
  if($value{machine}{cipuxMaster}{$key{'machine'}} eq "TRUE"){
    # isn:
    $key{'image_slot'} = $value{'machine'}{'cipuxImageSlotNumber'}{$machine};
    &log($SCRIPT,"- assign image slot [$key{'image_slot'}]");
    $key{'image_slot'} = substr("000".$value{'machine'}{'cipuxImageSlotNumber'}{$machine},-3,3);
    &log($SCRIPT,"- correct image slot [$key{'image_slot'}]");
  }else{# we use the room Image_Slot
     #$key{'image_slot'} = $key{'room'};
     $key{'image_slot'} = substr("000".$key{'room'},-3,3);
     # hack
     $value{'machine'}{'cipuxImageSlotNumber'}{$machine} = $key{'image_slot'};
  }
  #process images to calculate image number for client
  foreach my $image_number (keys %{$value{'image'}{'cipuxImageName'}}){
    &log($SCRIPT,"process image number. ($image_number)..........................");
    my $htn = $key{'hardware_type'};
    my $isn = $key{'image_slot'};
    my $r1=0;
    my $r2=0;
    $r1 = 1 if defined $value{'reference'}{'cipuxHardwareTypeNumber'}{$htn};
    $r2 = 1 if defined $value{'reference'}{'cipuxImageSlotNumber'}{$isn};
    &log($SCRIPT,"- htn($htn) isn($isn) r1($r1) r2($r2)");
    $key{'image'} = "";# no image exists for this machine
    if(defined($value{'reference'}{'cipuxHardwareTypeNumber'}{$image_number})){
      if(defined($value{'reference'}{'cipuxImageSlotNumber'}{$image_number})){
        my $ref_htn = $value{'reference'}{'cipuxHardwareTypeNumber'}{$image_number};
        my $ref_isn = $value{'reference'}{'cipuxImageSlotNumber'}{$image_number};
        &log($SCRIPT,"htn ($htn) eq ref_htn($ref_htn) and isn($isn) eq ref_isn($ref_isn)?");
        if($htn eq $ref_htn and $isn eq $ref_isn){
          &log($SCRIPT,"hit htn eq ref_htn and isn eq ref_isn");
          # means: an image for the client exists.
          $key{'image'} = $image_number;
          last;
        }
      }
    }
  }
  
  # replace every key in the template
  foreach my $key (keys %key){
      &log($SCRIPT,"Key $key value ($key{$key})");
      # process all machine, hardware_type, ... attributes
      if($key{$key} ne ""){# image may not be predefined as they should
                         # tinydeploy as to print error message
                         # that a user should define a virtual image in CAT
        foreach my $attribute (keys%{$value{$key}}){
          my $value = $value{$key}{$attribute}{$key{$key}};
          if(defined($value)){
            &log($SCRIPT,"- X attribute:             ($attribute) => value ($value)");
	    if(defined($value)and defined($attribute)){
            $tmp_tpl =~s#<SE::$attribute>#$value#g;
            }else{
	      print "ERROR $attribute ($value)\n";
	    }
         }
      }
    }
  }

  &log($SCRIPT,"- room [$key{room}]");
  # new short name
  my $short = $machine;
  $short =~s/cipworx_cipux_client/C/;
  &log($SCRIPT,"Extracted short: $short");
  $short =~s/_(\d+)\$$//g;
  my $string = substr("000".$1,-3,3);
  $short=$short."_".$string;
  &log($SCRIPT,"New short name with leading zeros: '$short'");
  $tmp_tpl=~s#<SE::SHORT>#$short#g;

  # CIPUX_IMAGE
  my $image_path = "";
  if($value{machine}{cipuxMaster}{$key{'machine'}} eq "TRUE"){
    &log($SCRIPT,"* master machine using image slot from client");
    # this sould be the full qualified name to the image without file extension
    # the image will be saved under the slot number
    my $string = substr("000".$key{'image_slot'},-3,3);
    $image_path = "cipux/image/$key{'hardware_type'}/$string";
    &log($SCRIPT,"  => image path [$image_path]");
  }else{
    &log($SCRIPT,"* restore machine using image slot from room [$key{room}]");
    my $string = substr("000".$key{'room'},-3,3);
    $image_path = "cipux/image/$key{'hardware_type'}/$string";
    &log($SCRIPT,"  => image path [$image_path]");
  }
  &log($SCRIPT,"-> CIPUX_IMAGE [$image_path]");
  
 # if(defined($value{'image'}{'cipuxImageType'}{$key{'image'}})){# type is aviable
 #   my $type = $value{'image'}{'cipuxImageType'}{$key{'image'}};
 #   &log($SCRIPT,"type: ($type)");
 #   $image_path= $image_path.'.'.$type;
 # }
  $tmp_tpl=~s#<SE::CIPUX_IMAGE>#$image_path#g;

  # write file name
  my $file_name = $dir."/".$short.".conf";
  &log($SCRIPT,"* write file [$file_name]");
  open(FILE,">$file_name") or die "Can't write to file '$file_name'!";
    print FILE $tmp_tpl;
  close(FILE);
  my $cmd = "chmod 644 $file_name";
  `$cmd`;
}

exit(0);


# +========================================================================+
$PART=              "Subroutines";#                                        |
# +========================================================================+

=pod

=head1 FUNCTIONS

=cut


# .........................................................................
# :                               usage                                   :
# .........................................................................
#
=pod

=head2 usage

Prints usage message.

=cut

sub usage{

  print "-"x76;
  print "\n";
  print "($BRAND-$PROJECT-$SCRIPT)\n\n";
  print "Creates shell scripts, which will be fetched by cipux clients. One\n";
  print "shell script for each client. This shell script tells a client what\n";
  print "image he should load.\n\n";
  print "USAGE:\n";
  print "       $SCRIPT      :                                  \n\n";
  print "       $SCRIPT -h   : prints this help message\n";
  print "\n";
  print "-"x76;
  print "\n";  

}	

#===========================================================================
#==== Documentation
#===========================================================================

=head1 SEE ALSO

See the CipUX webpage and the  manual at L<http://www.cipux.org>
See the mailing list L<http://www.cipux.org/mailman/listinfo/>

=head1 AUTHOR

Christian Kuelker  E<lt>christian.kuelker@cipworx.orgE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 1999-2005 by Christian Kuelker 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
MA 02110-1301, USA.

=cut
