#!/usr/bin/perl -w
# *========================================================================+
# || /usr/lib/cipux/bot/cipux_bot_dhcpd                                   ||
# ||                                                                      ||
# || This script have several purposes.                                   ||
# || (1) create a dhcpd.conf OR                                           ||
# || (2) writes a part of the dhcpd.conf for hosts OR                     ||
# || (3) writes an include for the dhcpd.conf for a subclass              ||
# ||                                                                      ||
# || Copyright: (c) 1999 - 2006  by Christian Kuelker all Rights reserved!||
# || Licence:   GPLv2                                                     ||
# ||                                                                      ||
# ||                                                                      ||
# *========================================================================+

# History:
# Revision ckuelker: 2006-08-16 adopted multible ltsp server
# Revision ckuelker: 2006-08-17 changed master.all, restore.all
# Revision ckuelker: 2006-08-20 add thinfat.all
# Revision ckuelker: 2006-08.25 change path of cipux_get_value

=pod

=head1 NAME

cipux_bot_dhcpd - Version <[CIPUX::VERSION]>  2006-08-20 CipUX 

=head1 SYNOPSIS

	cipux_bot_dhcpd [OPTIONS] -f dhcpd.conf 

        (1)  -f        :points to a file, the dhcpd.conf or an included part

	Auxiliary options:
        (2)  -t        :use dhcpd.tpl (if creating a hole dhcpd.conf)
        (*)  -a        :use dhcpd_client_host.tpl (for host entries)
        (4)  -c        :use dhcpd_class.tpl (for class entries)
        (5)  -s        :use dhcpd_client_subclass.tpl (for subclass entries)
        (*)  -n        :use dhcpd_subnet.tpl (for subnet/shared networks by room)
        (7)  -h        :get help message

         * to be implemented!

=head1 ABSTRACT

This script read the LDAP server and writes a new dhcpd.conf or a new part of
the dhcpd.conf

Two usage scenario can be seen:

(1) Create a hole dhcpd.conf

(2) Create a part dhcpd.conf, eg: cipux_subclass.conf

In either case use the -f option to point to the appropriate file.

Gernerally, to build a valid dhcpd.conf the options -c (host entries) and
-s (sublcass entries) should NOT be mixed.

The -s mode may be used to integrate cipux into other dhcpd setups, but
the clients won't get a fixed address from the subclass. Due to this fact 
there may be a lack of security or stability. One can not ensure that for 
example client C_Room-10_200 would become ip 10.10.118.200.

=head1 EXAMPLE

(1) cipux_bot_dhcpd -t -n -a -f /etc/cipux/dhcpd/dhcpd.conf

write a hole dhcpd.conf made from the dhcpd.tpl file. Then 
shared network, subnet and a host list will be added.

(2) cipux_bot_dhcpd -c -s -f /etc/cipux/dhcpd/cipux_subclass.conf

writes a cipux class and the appropriate subclass entries in
a file (witch will included by some other dhcpd.conf, probably)

=head1 REQUIREMENTS

       CipUX::CiBot
       cipux_get_value
       /etc/cipux/dhcpd/dhcpd.tpl
       /etc/cipux/dhcpd/dhcpd_client_host.tpl
       /etc/cipux/dhcpd/dhcpd_client_subclass.tpl

=head1 FILES

=head2 dhcpd.tpl

Here an example of a dhcpd.tpl. This have to changed to your need probably. 
Note that "use-host-decl-names on;" should NOT be used with the -s flag,
because it will be set there. But you may find a usable setting.

  # +=======================================================================+
  # || /etc/cipux/dhcpd/dhcpd.tpl                                          ||
  # || (auto generated from dhcpd.tpl, by cipux_bot_dhcpd)                 ||
  # ||                                                                     ||
  # ||                                                                     ||
  # || (c) Copyright 2002. 2003 by Christian Kuelker                       ||
  # || License: GPLv2                                                      ||
  # ||                                                                     ||
  # ||                                                                     ||
  # +=======================================================================+
  ddns-update-style none;
  ignore client-updates;

  # this we need for the pxegrub config file option
  option bootmenu code 150 = string;

  # thats what we are
  option domain-name "cipux.cipworx.org";
  option nis-domain  "cipux.cipworx.org";

  # Rudolf say, xxx do not need DNS nameserver (mail, april 2003)
  #option domain-name-servers 129.70.5.16,129.70.4.55;

  # where to get out of here
  option routers                  10.10.118.254;
  option subnet-mask              255.255.255.0;

  # what to do next (if not given use dhcpd server IP as tftpd server)
  #next-server server.cipux.cipworx.org;

  # how long we are graceful:
  max-lease-time 8600;
  default-lease-time 7200;

  # use with cipux_bot_dhcpd -t -n -a flag
  # comment this out with -c -s flag
  use-host-decl-names on;

  # knows anybody around here the time?
  # option time-offset -18000; # Eastern Standard Time

  # we are secure
  deny unknown-clients;

  # we are admins, so we know what we are doing
  authoritative;

  # lpr
  option lpr-servers              cipux;

  # bootp etherboot
  #allow booting;
  #allow bootp;

  # time Server
  #option ntp-servers              cipux;

  # netbios, Wins
  option netbios-node-type        8;      # Hybrid lookups (WINS, then broadcast)
  option netbios-name-servers     cipux;
  option netbios-dd-server        cipux;
  option netbios-scope            "";

=head2 dhcpd_subnet.tpl

  FILE NOT IMPLEMENTED JET

=head2 dhcpd_client_host.tpl

  FILE NOT IMPLEMENTED JET

=head2 dhcpd_class.tpl

  class "cipux_clients" {
    match hardware;
  }

=head2 dhcpd_client_subclass.tpl

  # host <[DHCPD_BOT::CLIENT_NUMBER]>
  subclass "cipux_clients" 1:<[DHCPD_BOT::MAC_ADDRESS]> {
    if substring (option vendor-class-identifier, 0, 7) = "cipworx" {
      use-host-decl-names off;
      option domain-name "<[DHCPD_BOT::CUSTOMER_SHORT]>.cipworx.de";
      option host-name "<[DHCPD_BOT::CLIENT_NAME]>";
    }else{
      use-host-decl-names on;
    }
  }

=cut

package cipux_bot_dhcpd;

# +========================================================================+
my $PART=                    "Global Part";#                               |
# +========================================================================+

use CipUX::CiBot;
use CipUX::Task;
use strict;

my $BRAND   = "cipworx";
my $PROJECT = "cipux";
my $SCRIPT  = "cipux_bot_dhcpd";
my $GET     = "/usr/lib/cipux/lal/cipux_get_value";
my $Customer = $cipux{Customer};

&cibot_get_ldap_config();

# +========================================================================+
$PART=                       "Configuration";#                             |
# +========================================================================+

#my $cibot_href = &read_ldap_conf_obj('cibot');
#$DEBUG = $$cibot_href{'Cibot_Debug'} if defined($$cibot_href{'Cibot_Debug'});
#&do_log_start;

#my $cipux_href = &read_ldap_conf_obj('cipux');

&log($SCRIPT,"="x76);

# +========================================================================+
$PART=                       "Configuration";#                             |
# +========================================================================+

#my $slapd_path = "/usr/local/libexec/slapd";
#my @ps = `ps ax|grep slapd`;
#my $running = 0;
#
#foreach(@ps){
#  chomp();
#  $running = 1 if m/$slapd_path/
#}
#if($running){
#  &log($SCRIPT,"LDAP is running!");
#}else{
#  print "LDAP is not running!\n";
#  exit(1);
#}

# +========================================================================+
$PART=                       "Environment";#                               |
# +========================================================================+

# use of environment module
use Getopt::Std;
my %opt=();
getopts("hf:tacsn",\%opt);

if(defined($opt{h})){usage();exit(0)}
if(not defined($opt{f})){usage();exit(0)}
my $file = $opt{f};

my %tpl_file = ();
my $p = "/etc/cipux/dhcpd/dhcpd";
$tpl_file{dhcpd}                 = $p.".tpl"                 if defined($opt{t});
$tpl_file{dhcpd_class}           = $p."_class.tpl"           if defined($opt{c});
$tpl_file{dhcpd_subnet}          = $p."_subnet.tpl"          if defined($opt{n});
$tpl_file{dhcpd_client_host}     = $p."_client_host.tpl"     if defined($opt{a});
$tpl_file{dhcpd_client_subclass} = $p."_client_subclass.tpl" if defined($opt{s});

&log($SCRIPT,"option");
#if($DEBUG){
#  foreach(keys%tpl_file){&log($SCRIPT,"got from option file [$_][$tpl_file{$_}]")}
#}

# +========================================================================+
$PART=                       "Main";#                                      |
# +========================================================================+

my @tpl=();

# we process the base tpl file oly if we have -t option
if(defined($tpl_file{dhcpd})){
  # we load the tpl file
  if(-e $tpl_file{dhcpd}){
    &log($SCRIPT,"found dhcpd.tpl file ($tpl_file{dhcpd})");
    my $cmd = "cat $tpl_file{dhcpd}";
    &log($SCRIPT,"CMD ($cmd)");
    @tpl = `$cmd`;
  }else{
    &log($SCRIPT,"ERROR: dhcpd.pl file ($tpl_file{dhcpd}) not found!, exiting");
    exit(1);
  }
}

# test for LDAP and data
my $command = "$GET -e 'cipuxNetworkAddress' -R";
&log($SCRIPT,"CMD ($command)");
my @room =  `$command`;

if(defined($room[0])){
  if($room[0] =~/ERROR/){#LDAP might be up
    my $message ="Can't bind to LDAP server! Check connection and setup!";
    &log($SCRIPT,"ERROR 0155: $message");
    # Can't bind to LDAP server! Check connection and setup!
    error('0155',$SCRIPT); 
    exit(1);
  }
}else{# LDAP down or now rooms!
  my $message = "Can't bind to LDAP server or no rooms defined!";
  &log($SCRIPT,"ERROR 0177: $message"); 
  # Can't bind to LDAP server or no rooms defined!
  error('0177',$SCRIPT); 
  exit(1);
}

my %room;
foreach (@room){
  chomp();
  my($key,$value)=split("\t");
  $room{$key}=$value;
}

my @command=();

my @attr = qw(
              cipuxRoomName
              cipuxClientNumber
              cipuxMacAddress
              cipuxMaster
              cipuxImageNumber
             );

my %command = ();	     
foreach(@attr){
  $command{$_}="$GET -e '$_' -M";
}

# HoH (Perl <284>)
my %dhcp = ();
my %tmp = ();
foreach my $c (sort keys %command){
  &log($SCRIPT,"------------------------------------------------------------");
  &log($SCRIPT,"CMD($command{$c})");
  foreach (`$command{$c}`){
    chomp();
    my($key,$value)=split("\t");
    #&log($SCRIPT,"image($_)")
    &log($SCRIPT,"- key($key)=>($value)");
    $tmp{$key}=$value;
  }
  $dhcp{$c} = { %tmp };
}

# HoH (Perl <284>)
#if($DEBUG){
#  foreach $attr (sort keys %ldap){
#    &log($SCRIPT,"------------------------------------------------------------");
#    &log($SCRIPT,"attribute($attr) in hashref($dhcp{$attr})");
#    foreach $client (sort keys %{ $dhcp{$attr} }){
#      &log($SCRIPT,"- client($client)=>($dhcp{$attr}{$client})");
#    }
#  }
#  foreach (sort keys %room){
#    &log($SCRIPT,"room($_)=>($room{$_})");
#  }
#}

# +========================================================================+
$PART=              "Create dhcpd conf";#                                  |
# +========================================================================+

### Identifizierung:
###
### Option fuer Bootprom v2.00
### Fuer Bootprom v2.20 darf die Option (Tag60):
  #
  # option dhcp-class-identifier "PXEClient";
  #
### nicht eingeschaltet werden, da der PXE-Client sonst auf
### Port 4011 anfragt und einen ProxyDHCP Server erwartet.
###
### Vendor Spezifikation:
###
  #
  # option vendor-encapsulated-options 01:04:00:00:00:00;

#  host cipworx_cipux_Raum-o51_41 {
#     hardware ethernet 00:04:76:20:32:80;
#     fixed-address 10.10.118.41;
#     option host-name "client41";
#     filename "/cipux/boot/restore.all";
#     option option-150 "(hd0,0)/boot.cfg"; # v2
#     option bootmenu   "(hd0,0)/boot.cfg"; # v3
#  }


# if option -n (shared net and subnet) 
# we process shared network and subnet
if(defined($tpl_file{dhcpd_subnet})){

  #write out shared-network cipuxnet {
  push(@tpl,"shared-network cipuxnet { \n");

  # write out subnets
  foreach my $r (sort keys %room){
    # TODO  netmask values into LDAP
    push(@tpl,"\n  subnet $room{$r} netmask 255.255.255.0 {\n");
    push(@tpl,"    option routers ".&cipux_task_tell_me_room_gateway_address('cgi',$r).";");
    my $range_start = cipux_task_tell_me_room_network_address('cgi',$r);
    my $range_end = $range_start;
    # TODO range values into LDAP!
    $range_start =~ s/\.\d+$/\.241/;
    $range_end =~ s/\.\d+$/\.253/;
    #push(@tpl,"    range $range_start $range_end  ;");
    push(@tpl,"  }");
  }

  # make a cipux group
  push(@tpl,"\n  group cipux {");
  push(@tpl,"    default-lease-time -1;");
  push(@tpl,"    option vendor-encapsulated-options 01:04:00:00:00:00;");
  push(@tpl,"\n  ");
}

# if option -a, we write a hostlist
if(defined($tpl_file{dhcpd_client_host})){

  # write out a host list
  foreach my $client (sort keys %{ $dhcp{'cipuxRoomName'} }){
    &log($SCRIPT,"--------------------------------------------------------------");
    &log($SCRIPT,"* client  ($client)");
    &log($SCRIPT,"* MAC addr($dhcp{'cipuxMacAddress'}{$client})");
    my $host = $client;
    $host =~ s/\$+$//g; # win clients might have it 	  
    push(@tpl,"    host $host {");
    push(@tpl,"      hardware ethernet $dhcp{'cipuxMacAddress'}{$client};");
    my $filename = "filename \"/cipux/boot/";
    if($dhcp{'cipuxMaster'}{$client} eq "TRUE"){
      $filename .= $ldap{CipUX}{CipUX_Bootloader_Master}.'.'.$ldap{CipUX}{CipUX_Bootloader_Nic}.'";';
    }else{
      $filename .= $ldap{CipUX}{CipUX_Bootloader_Restore}.'.'.$ldap{CipUX}{CipUX_Bootloader_Nic}.'";';
      if($ldap{CipUX}{CipUX_Bootmode} eq "auto"){
        push(@tpl,"      option bootmenu \"(hd0,0)/autoboot.cfg\";");
      }else{
        push(@tpl,"      option bootmenu \"(hd0,0)/demandboot.cfg\";");
      }
    }
    &log($SCRIPT,"* filename($filename)");
    #something to extract the client number
    my $ord = &ip_ord($room{$dhcp{'cipuxRoomName'}{$client}},$client,1);
    my $ip  = &ip_ord($room{$dhcp{'cipuxRoomName'}{$client}},$client,2);

    push(@tpl,"      fixed-address $ip;");
    push(@tpl,"      option host-name \"C_$dhcp{'cipuxRoomName'}{$client}_$ord\";");
    push(@tpl,"      $filename");
    push(@tpl,"    }");
  }
  push(@tpl,"  }\n");
}

# if option -n (shared net and subnet) 
# we process shared network and subnet
if(defined($tpl_file{dhcpd_subnet})){

  #close shared-network cipuxnet {
  push(@tpl,"}\n");
}

# if option -c, write a class from dhcpd_class.tpl
if(defined($tpl_file{dhcpd_class})){
 
  if(-e $tpl_file{dhcpd_class}){
    &log($SCRIPT,"open template file [$tpl_file{dhcpd_class}]");
    my $cmd = "cat $tpl_file{dhcpd_class}";
    &log($SCRIPT,"CMD [$cmd]");
    my $class = `$cmd`;
    push(@tpl,$class);
  }else{
    my $message = "Can't open template file for dhcpd class:";
    &log($SCRIPT,"ERROR: $message [$tpl_file{dhcpd_class}]!");
    # Can't open template file for dhcpd class:
    &error('0180',$tpl_file{dhcpd_class});
    exit(1);
  }
}

# if option -s, write a subclass from dhcpd_client_subclass.tpl
if(defined($tpl_file{dhcpd_client_subclass})){

  if(-e $tpl_file{dhcpd_client_subclass}){

    &log($SCRIPT,"open template file [$tpl_file{dhcpd_client_subclass}]");
    my $cmd = "cat $tpl_file{dhcpd_client_subclass}";
    &log($SCRIPT,"CMD [$cmd]");
    my $subclass = `$cmd`;

    # write out a client subclass list
    foreach my $client (sort keys %{ $dhcp{'cipuxRoomName'} }){
      &log($SCRIPT,"client [$client]");
      my $string = $subclass;
      
      #something to extract the client number
      my $ord = &ip_ord($room{$dhcp{'cipuxRoomName'}{$client}},$client,1);
      my $client_name = "C_$dhcp{'cipuxRoomName'}{$client}_$ord";
      $string =~ s/<\[DHCPD_BOT::CLIENT_NAME\]>/$client_name/g; 
      $string =~ s/<\[DHCPD_BOT::CLIENT_NUMBER\]>/$ord/g; 
      $string =~ s/<\[DHCPD_BOT::CUSTOMER_SHORT\]>/$Customer/g; 
      $string =~ s/<\[DHCPD_BOT::MAC_ADDRESS\]>/$dhcp{'cipuxMacAddress'}{$client}/g; 
      push(@tpl,$string);
    }
  }else{
    my $message = "Can't open template file for dhcpd client subclass:";
    &log($SCRIPT,"ERROR: $message [$tpl_file{dhcpd_class}]!");
    # Can't open template file for dhcpd client subclass:
    &error('0181',$tpl_file{dhcpd_client_subclass});
    exit(1);
  }
}


# +========================================================================+
$PART=              "Write dhcpd conf";#                                   |
# +========================================================================+

open(CONF,"> $file") or die "can't write to '$file'\n";
foreach(@tpl){
  chomp();
  print CONF "$_\n";
}
close(CONF);


# +========================================================================+
$PART=              "Subroutines";#                                        |
# +========================================================================+

=pod

=head1 FUNCTIONS

=cut





# .........................................................................
# :                               ip_ord                                  :
# .........................................................................

=pod

=head2 ip_sub

deliver client number (ord) or ip address.

<b>USAGE:</b>

  $ord = &ip_sub(X,client,1);
  $ip  = &ip_ord(X,client,2);

=cut

sub ip_ord{
  my $ip     = shift;
  my $client = shift;
  my $mode   = shift;
  &log($SCRIPT," ip_ord($ip,$client,$mode)");
  &log($SCRIPT,"WARNING: got NOT defined mode, some error in calling &ip_sub in dhcpd_bod.pl") if not defined($mode);
  $mode = 1 if not defined($mode);
  $ip =~ s/0$/$dhcp{'cipuxClientNumber'}{$client}/;
  my $ord=substr("00".$dhcp{'cipuxClientNumber'}{$client},-3,4);
  return $ord if $mode == 1;
  return $ip if $mode == 2;

}

# .........................................................................
# :                               usage                                   :
# .........................................................................

=pod

=head2 usage

Prints usage message.

=cut

sub usage{
  print "-"x76;
  print "\n";
  print "USAGE: ($BRAND-$PROJECT-$SCRIPT)\n";
  print "       $SCRIPT [OPTIONS] -f dhcpd.conf\n";
  print "\n";
  print " (1)  -f        :points to a file, the dhcpd.conf or an included part\n";
  print "\n";        
  print "	Auxiliary options:\n";
  print " (2)  -t        :use dhcpd.tpl (if creating a hole dhcpd.conf)\n";
  print " (*)  -a        :use dhcpd_client_host.tpl (for host entries)\n";
  print " (4)  -c        :use dhcpd_class.tpl (for class entries)\n";
  print " (5)  -s        :use dhcpd_client_subclass.tpl (for subclass entries)\n";
  print " (*)  -n        :use dhcpd_subnet.tpl (for subnet/shared networks by room)\n";
  print " (7)  -h        :get help message\n";
  print "\n";
  print " * file parse not implemented jet, but entries will be created.\n";
  print "\n";
  print "Examples:\n";
  print " (1) cipux_bot_dhcpd -t -n -a -f /etc/cipux/dhcpd/dhcpd.conf\n";
  print "     write a hole dhcpd.conf made from the dhcpd.tpl file. Then\n"; 
  print "     shared network, subnet and a host list will be added.\n";
  print " (2) cipux_bot_dhcpd -c -s -f /etc/cipux/dhcpd/cipux_subclass.conf\n";
  print "     writes a cipux class and the appropriate subclass entries in\n";
  print "     a file (witch will included by some other dhcpd.conf, probably)\n";
  print "-"x76;
  print "\n";
}

#===========================================================================
#==== Documentation ========================================================
#===========================================================================

=head1 SEE ALSO

See the CipUX webpage and the  manual at L<http://www.cipux.org>
See the mailing list L<http://www.cipux.org/mailman/listinfo/>

=head1 AUTHOR

Christian Kuelker  E<lt>christian.kuelker@cipworx.orgE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 1999-2005 by Christian Kuelker 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
MA 02110-1301, USA.

=cut
