#!/usr/bin/perl -w
# *========================================================================+
# || <[CIBOT::DIR]>/bin/image_bot.pl
# || Version <[CIPUX::VERSION]>
# || 
# || This script parses the tftpboot dir and collect the image values.
# || The value will be stored into the LDAP server.
# ||
# ||
# ||
# || Copyright: (c) 1999 - 2005  by Christian Kuelker all Rights reserved!
# || Licence:   GPLv2
# || 
# || Revision:  2003-08-14
# || Build:     "<[CIBOT::BUILD]>";
# *========================================================================+

=pod

=head1 NAME

image_bot.pl - Version <[CIPUX::VERSION]>  2003-04-07 CipUX

=head1 SYNOPSIS

	image_bot.pl [OPTIONS] 

	Auxiliary options:
        (1)  -h        :get help message

=head1 ABSTRACT

This script parses the tftpboot dir and collect the image values.
The value will be stored into the LDAP server.

=head1 REQUIREMENTS

  cibot.pm
  File::Find;
  Cwd

=cut

package image_bot;

use CipUX::CiBot;
use Getopt::Std;
use File::Find;
use Cwd;
use strict;

# +========================================================================+
my $PART=                    "Global Part";#                               |
# +========================================================================+
my $BRAND   = "cipworx";
my $PROJECT = "cipux";
my $SCRIPT  = "cipux_bot_image";

#my $BRAND   = "cipworx";
#my $PROJECT = "cipux";
#my $SCRIPT  = "image_bot.pl";
my $VERSION = "<[CIPUX::VERSION]>";
my $BUILD   = "<[CIBOT::BUILD]>";
my $DEBUG   = 0;

my $CLEANSING =0;# if =1, will remove files
my $GET = "/usr/lib/cipux/lal/cipux_get_value";

# +========================================================================+
$PART=                       "Environment";#                               |
# +========================================================================+

# use of environment module
my %opt=();
getopts("h",\%opt);

if(defined($opt{h})){usage();exit(0)}

# +========================================================================+
$PART=                       "Configuration";#                             |
# +========================================================================+


&log($SCRIPT,"="x76);

# Date as ISO-8601
# #$date = actual_date();
#
# # Time
# #my $time = actual_time();
 my $time = qx(date);
 &log($SCRIPT,"Time = $time");


&cibot_get_ldap_config();

#my $cibot_href = &read_ldap_conf_obj('cibot');
$DEBUG = $ldap{CipUX}{'Cibot_Debug'} if defined($ldap{CipUX}{'Cibot_Debug'});

#+========================================================================+
$PART=                       "Var";#                                      |
#+========================================================================+

# HoA Perl <279>
my %attribute = (
   i => [qw(cipuxImageName cipuxImageType cipuxChecksum cipuxImageSize cn)],
                );
my %key = (
   i => 'cn'
          );

#+========================================================================+
$PART=                       "Prepare";#                                  |
#+========================================================================+

# store all image into a HoH, to minimize write to DB
my %data=();
&log($SCRIPT,"prepare:");
foreach my $option(keys %attribute){
  &log($SCRIPT,"-----------------------------------------------------------");
  &log($SCRIPT,"option($option)");
  foreach my $attribute(@{$attribute{$option}}){
    if($attribute ne $key{$option}){# overdue!!! (to collect cn)
      &log($SCRIPT,"-------------------------------------------------------------");
      &log($SCRIPT,"attribute($attribute)");
      my $opt=uc($option);
      my @data = qx($GET -$opt -e $attribute);
      foreach (@data){
        chomp();
        my($key,$value)=split("\t");
        $data{$option}{$attribute}{$key}=$value;
        &log($SCRIPT,"option($option)attribute($attribute)key($key)=>($value)");
      }
    }
  }
}

#+========================================================================+
$PART=                       "Main";#                                     |
#+========================================================================+

my $tftpdir = $ldap{CipUX}{Cibot_Tftpboot_Dir};
$tftpdir ="/var/lib/tftpboot/cipux";

&log($SCRIPT,"tftpdir($tftpdir)");
my %file_matrix=();
my %put=();

my %ht=();
foreach('001'..'999'){$ht{$_}=1}

if(-d "$tftpdir/image"){
  chdir "$tftpdir/image";
  find(\&process_image_dir, '.');# Cookbook 9.11 <346>
}else{
  print STDER "ERROR: tfttpdir NOT valid ($tftpdir/image)\n";
  &log($SCRIPT,"
  ========================================================
  ERROR:  tfttpdir NOT valid ($tftpdir/image), exiting!
  ACTION: set a valid in configuration -> cipbot 
          like: Cibot_Tftpboot_Dir=/tftpboot/cipux
          as LDAP attribute cipuxVariable!
  ========================================================
  ");
  exit(1);
}

&log($SCRIPT,"#");
&log($SCRIPT,"# process recieved files:");
&log($SCRIPT,"#");
# process the recieved files
foreach my $dir (keys %file_matrix){ # $dir = 001, 002 003 004 ...

  # sum up file for log
  if($DEBUG){
    my $v = "";
    foreach(keys%{$file_matrix{$dir}}){$v.="[".$_."]"}
    &log($SCRIPT,"dir($dir)=>($v)");
  }

  # for each file in that dir
  foreach my $file (keys%{$file_matrix{$dir}}){
    # $file = 001/001.tgz, 001/001.cfg, 001/001.dd, 001/002.tgz, ...
    &log($SCRIPT,"* process file ($file)");
    if($file =~m/\.cfg$/){ # its this a file with .cfg extension
                           # if so, it MIGHT be a valid cfg file!
      my $cfg = $file;
      $cfg=~s/\.cfg$//g;
      $cfg=~s#$dir/##g;
      &log($SCRIPT,"  -> this might be a valid cfg file ($file)=>(cfg)");
      if(defined($ht{$cfg})){ # 001..999 ? Check for valit cfg file
        &log($SCRIPT,"  -> this is (!) a valid cfg file ($file)=>($cfg)");
        &log($SCRIPT,"     collect export data for the DB ...");
        #
        my @out = qx(cat $file|grep export);
        foreach my $line (@out){# get the mighty cfg file
          chomp($line);
          &log($SCRIPT,"     -> got line ($line)");
          if($line=~ m/export/){
            my($string,$value) = split("=",$line);
            my($string2,$attribute) = split(" ",$string);
            $value=~s/'//g;
            $value=~s/^\s+|\s+$//g;# del spaces
            $attribute=~s/^\s+|\s+$//g;# del spaces
            &log($SCRIPT,"        store DB attribute($attribute)=>($value)");
            $put{$attribute}=$value;
          }
        }
        # after we got all attributes we calculate a hash
        if(defined($put{cn})){# ok we have a image number
          &log($SCRIPT,"     -> got image number ($put{cn})");
          foreach my $option(keys %attribute){
            &log($SCRIPT,"        .............................................");
            &log($SCRIPT,"        option   ($option)");
            foreach my $attribute(@{$attribute{$option}}){
              if($attribute ne $key{$option}){
                # if we process the 'cn' attribute, we
                # can get into deep trouble, because of the risk of
                # mooving LDAP entries!!
                if(defined($put{$attribute})){
                  # some values might by empty: due to murks ;-)
                  my $data = $data{$option}{$attribute}{$put{cn}};
                  my $new  = $put{$attribute};# may be new
                  &log($SCRIPT,"        attribute($attribute)");
                  &log($SCRIPT,"        key      ($put{cn})=>($new)");
                  &log($SCRIPT,"        ($data) = ($new)?");
                  if($data ne $new){# we got new data!
                    &log($SCRIPT,"        -> new data ($new)");
                    my $cmd = "cipux_set_value"
                            . " -$option '$put{cn}'"
                            . " -e '$attribute'"
                            . " -v '$new'";             
                    &log($SCRIPT,"           CMD($cmd)");
		    # TODO write image size in 001.cfg
		    if($new eq "<SE::cipuxImageSize>"){
                      # skip that for now, SE::cipuxImageSize not used
		    }else{
                      my @out = qx($cmd);
                      foreach(@out){
                        chomp();
                        print STDER "ERROR:($SCRIPT) $_";
                      }
		    }
                  }
                }else{# a not defined attribute occured!
                      # this meas we WANT to have it, BUT
                      # the 'create image script' from murks didn't
                      # gave us, bad!
                  &log($SCRIPT,"
  ========================================================================
  WARNING: a cibot error occured. There should be a LDAP
           attribute ($attribute) but it was not found in
           the config file. This should be fixed! The
           reason could be: (1) The value was not created
           by deploy_conf_bot.pl or (2) it was not created
           by TinyDeploy 4.sh
  ACTION:  Call the the 3rd or 4th level support.
  ========================================================================
                         ");
                } # empty or non empty value
              } # no cn ?
            }
          }
        }else{# sorry, no image number forget it
         &log($SCRIPT,"
  ===============================================================
  WARNING: got empty image number!
  ACTION:  Can't do anything!
  ===============================================================
                ");
        }
      }else{
         &log($SCRIPT,"  -> Well, this is NOT a valid cfg file ($file)");
         &log($SCRIPT,"     It has the wrong name!");
         &log($SCRIPT,"
   ======================================================
   WARNING: found NON valid *.cfg file! ($file)
   ACTION:  Will cleanse the mighty cfg file
            if CLEANSING is active ($CLEANSING)
   ======================================================
              ");
        qx(rm -f $cfg) if ($CLEANSING);
      }
    }else{
      # search for the cfg files  
      &log($SCRIPT,"  -> this is NOT a cfg file ");
      &log($SCRIPT,"    ($dir) fm($dir)($file)($file_matrix{$dir}{$file})");
      # TODO find out if its a valid image file?
      # if no kill it
    }
  }
}



exit(0);


# +========================================================================+
$PART=              "Subroutines";#                                        |
# +========================================================================+

=pod

=head1 FUNCTIONS

=cut




# .........................................................................
# :                               do_log                                  :
# .........................................................................

=pod

=head2 do_log

Prints log messages using cibot.pm log function.

=cut

sub do_log{
  my $message = shift;
  $PART = "Log" if $PART eq "";
  my $logfile="/var/log/cipux/";
  if(!-d $logfile){mkdir($logfile,700)}
  $logfile="/var/log/cipux/$SCRIPT.log";

  #print "logfile $logfile\n";
  &log($SCRIPT,$PART,$logfile,$message) if $DEBUG;
}

# .........................................................................
# :                               do_log_start                            :
# .........................................................................

=pod

=head2 do_log_start

Prints log init message with do_log function.

=cut

sub do_log_start{
  &log($SCRIPT,
   "======================================================================"
         );
  $BUILD = "unknown"  if $BUILD eq "";
  &log($SCRIPT,"BEGIN LOG: $SCRIPT BUILD: $BUILD DEBUG_LEVEL: $DEBUG");
}



# .........................................................................
# :                               process_image_dir                       :
# .........................................................................

sub process_image_dir{
  my ($dev,$ino,$mode)=lstat($_);
  my $name = $File::Find::name;
  $name =~ s!^\./!!;
  my $dir = $File::Find::dir;
  $dir =~ s!^\./!!;
  $ht{$name}=0 if not defined ($ht{$name});
  #  &log($SCRIPT,"dir($dir) name($name) dev($dev) ino($ino) mode($mode) ht($ht{$name})");
  if($dir ne '.'){# a file
    &log($SCRIPT,"-------------------------------------------------------------");
    &log($SCRIPT,"found file in dir($dir)");
    &log($SCRIPT,"name($name) dev($dev) ino($ino) mode($mode) ht($ht{$dir})");

    if($ht{$dir}){# its a hardware type dir
      my $max = $ldap{CipUX}{CipUX_Hal_Hardware_Type_Max};
      &log($SCRIPT,"  * found member of hardware type in ($dir)($ht{$dir}), max($max)");
      if($dir <= $max){# ok its a valid image location
        &log($SCRIPT,"    - found valid image location fm($dir)($name)");
        $file_matrix{$dir}{$name}=1;
      }else{# not a valid image location, have to be killed
        &log($SCRIPT,"
        ===================================================
        WARNING: found NON valid image location!
        ACTION:  will try to clean it up, to kill it, to
                 sweep it out. (in a future release!)
        ===================================================
               ");
        # TODO write cleaning (or somewhere else!)
      }
    }
  }
}

# .........................................................................
# :                               usage                                   :
# .........................................................................

=pod

=head2 usage

Prints usage message.

=cut

sub usage{

#============================================================|

print "-"x76;
print "\n";
print "USAGE: ($BRAND-$PROJECT-$SCRIPT-$VERSION)\n";
print "       $SCRIPT [OPTIONS] \n";
print "Auxiliary options: \n";
print " (1)  -h        :this message\n";
print "-"x76;
print "\n";
}

=pod

=head1 AUTOR

Author Christian Kuelker <christian.kuelker@cipworx.org>

Copyright (c) 2003-04-07 Christian Kuelker

Published under the GNU Public License: GPLv2

=head1 BUGS

Not known.

=head1 SEE ALSO

ls, chmod

=cut

# TODO SECTION

# Write code for cleaning up the dirs
# (1) wrong name (other files)
# (2) dublicate cci and tgz image
# (3) no tgz/cci for cfg 
# (4) no cfg for tgz/cci


