#!/usr/bin/perl -w
# +=======================================================================+
# || cipux_rpc                                                           ||
# ||                                                                     ||
# || Testclient for cipux_rpcd                                           ||
# ||                                                                     ||
# ||                                                                     ||
# ||                                                                     ||
# ||                                                                     ||
# ||                                                                     ||
# || (c) Copyright 2006 by Christian Kuelker                             ||
# || All rights reserved!                                                ||
# ||                                                                     ||
# || License: GPLv2 or later                                             ||
# ||                                                                     ||
# +=======================================================================+
# history
# 2006-10-29 ckuelker:add logout test

use Frontier::Client;
use Term::ReadKey;

my $http_url = "http://127.0.0.1:8001/RPC2";
my $socket = Frontier::Client->new(url => $http_url);

# +=======================================================================+
# || test  the test function                                             ||
# +=======================================================================+

print "="x76,"\n";
print "(1) connect to $http_url locally ...\n";
print "    test if the socket server is there:\n";
my $status =  $socket->call('ping');
print "    the status of the server is $status\n";

# +=======================================================================+
# || log  in to the server                                               ||
# +=======================================================================+

# if you realy want to test the a remote connetction, change localhost
my $https_url = "https://localhost:8000/RPC2";
my $server = Frontier::Client->new(url => $https_url);

print "="x76,"\n";
print "(2) connect to $https_url over SSL ...\n";
print "    test if the https server is there:\n";
my $status2 =  $server->call('ping');
print "    the status of the server is $status2\n";

# +=======================================================================+
# || print sum                                                           ||
# +=======================================================================+

print "="x76,"\n";
print "(3) connect to $https_url over SSL ...\n";
print "    test the sum function:\n";
my @args=qw(3 4);
my $sum =  $server->call('sum',@args);
if( $$sum[0] eq '7'){
print "    the sum function is working ($$sum[0])\n";
}else{
print "    the sum function is NOT working ($$sum[0])\n";
}

# +=======================================================================+
# || login function                                                      ||
# +=======================================================================+

print "="x76,"\n";
print "(4) connect to $https_url over SSL ...\n";

ReadMode('normal');
print "    Give a valid login and password,\n";
print "    to log in to the CipUX XML-RPC server.\n\n";
print "    Login: ";
$login = ReadLine 0;
chomp($login);

ReadMode('noecho');
print "    Password: ";
$password = ReadLine 0;
chomp($password);
print "\n";

ReadMode('normal');
@args=();
push(@args,$login);
push(@args,$password);

my $ticket = $server->call('login',@args);
if(defined($ticket)){
  if($ticket ne ''){
    print "    OK, got secret ticket (you should not print that!)\n";
  }else{
    print "    wrong login or password!\n";
    exit(1);
  }
}else{
  print "    wrong login or password!\n";
  exit(1);
}

# +=======================================================================+
# || list users                                                          ||
# +=======================================================================+

print "="x76,"\n";
print "(5) connect to $https_url over SSL ...\n";
print "    try to get the userlist with cipux_task_list_users\n";
@args=();
push(@args,$login);
push(@args,$ticket);
my $user_ref = $server->call('cipux_task_list_users',@args);
foreach my $user (@$user_ref){
 print "    OK, got user: $user\n";
}


# +=======================================================================+
# || ttl                                                                 ||
# +=======================================================================+

print "="x76,"\n";
print "(6) connect to $https_url over SSL ...\n";
print "    try to get ttl\n";
my $ttl = $server->call('ttl',@{[$login,$ticket]});
print "    TTL = [$ttl] minutes\n";

# +=======================================================================+
# || session                                                             ||
# +=======================================================================+

print "="x76,"\n";
print "(7) connect to $https_url over SSL ...\n";
print "    try extend session ticket\n";
my $ticket2 = $server->call('session',@{[$login,$ticket]});
$ticket = $ticket2;

# +=======================================================================+
# || log out                                                             ||
# +=======================================================================+

print "="x76,"\n";
print "(8) connect to $https_url over SSL ...\n";
print "    try to log out\n";
my $result = $server->call('logout',@{[$login,$ticket]}) || 0;
if($result){
  print "    logout successful!\n";
}else{
  print "    logout not successful: wrong uid | wrong ticket | expired\n";
}

# +=======================================================================+
# || list users                                                          ||
# +=======================================================================+
print "="x76,"\n";
print "(9) connect to $https_url over SSL ...\n";
print "    try to get the userlist with cipux_task_list_users\n";
print "    this should NOT print a list, because we are logged out!\n";
@args=();
push(@args,$login);
push(@args,$ticket);
my $user_ref2 = $server->call('cipux_task_list_users',@args);
foreach my $user (@$user_ref2){
 print "    OK, got user: $user\n";
}



# +=======================================================================+
# || change_own_password                                                 ||
# +=======================================================================+
#
# print "="x76,"\n";
# print "(6) connect to $https_url over SSL ...\n";
# print "    try to get the userlist with cipux_task_change_own_password\n";
# @args=();
# push(@args,$login);
# push(@args,"password");
# $server->call('cipux_task_change_own_password',@args);

exit(0);





# +=======================================================================+
# || change_own_password                                                 ||
# +=======================================================================+
#
# print "="x76,"\n";
# print "(6) connect to $https_url over SSL ...\n";
# print "    try to get the userlist with cipux_task_change_own_password\n";
# @args=();
# push(@args,$login);
# push(@args,"password");
# $server->call('cipux_task_change_own_password',@args);

exit(0);




